#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : files
# @Time         : 2025/4/2 10:40
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo: 一般大文件问答需要
# https://ai.google.dev/gemini-api/docs/document-processing?hl=zh-cn&lang=python

from meutils.pipe import *

file = "/Users/betterme/PycharmProjects/AI/QR.png"


#
# file_object = client.files.upload(file=file)
# prompt = "一句话总结"

# file_object = client.aio.files.upload(file=file)


async def upload(self, files: Union[str, List[str]], client: Optional[genai.Client] = None):  # => openai files
    client = client or await self.get_client()

    if isinstance(files, list):
        return await asyncio.gather(*map(self.upload, files))

    file_config = {"name": f"{shortuuid.random().lower()}", "mime_type": guess_mime_type(files)}
    return await client.aio.files.upload(file=io.BytesIO(await to_bytes(files)), config=file_config)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : browser
# @Time         : 2023/11/29 15:52
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo: 持续刷新不掉线

from meutils.pipe import *
from meutils.decorators.retry import retrying
from meutils.playwright_utils.reload import refresh_cookies

from playwright.async_api import Playwright, async_playwright

cli = typer.Typer(name="模拟浏览器")


@retrying
async def deepseek_sign_in(  # todo
        user_file: str = '',
        headless: bool = False,
        timeout: int = 1000,
        **kwargs
):
    url = "https://chat.deepseek.com/sign_in"
    storages = []
    df = pd.read_csv(user_file, sep=' ', names=['user', 'password'])
    for _, user, password in df.itertuples():
        async with async_playwright() as playwright:
            browser = await playwright.chromium.launch(headless=headless)
            context = await browser.new_context()

            page = await context.new_page()

            ####################################################################################todo：linux模拟登录有问题

            await page.goto(url)
            await page.wait_for_load_state(state='load')
            await page.wait_for_load_state(state='networkidle')
            await page.wait_for_load_state(state='domcontentloaded')
            await page.wait_for_timeout(timeout=timeout)

            await page.get_by_placeholder("请输入手机号/邮箱地址").click()
            await page.get_by_placeholder("请输入手机号/邮箱地址").fill(user)
            await page.get_by_placeholder("请输入手机号/邮箱地址").press("Tab")
            await page.get_by_placeholder("请输入密码").fill(password)
            await page.get_by_role("img").nth(3).click()
            await page.get_by_role("button", name="登录").click()

            await page.wait_for_load_state(state='load')
            await page.wait_for_load_state(state='networkidle')
            await page.wait_for_load_state(state='domcontentloaded')
            await page.wait_for_timeout(timeout=timeout)

            # # 聊天
            await page.get_by_placeholder("请输入问题。Enter 发送，Ctrl + Enter 换行").fill("你是谁")
            await page.get_by_role("button", name="发送").click()

            ####################################################################################

            # ---------------------
            # 保存状态文件
            storage_state = f'{Path(user_file).parent}/deepseek_{user}.json'
            storage = await context.storage_state(path=storage_state)  # 覆盖更新
            storages.append(storage)
            await context.close()
            await browser.close()

    return storages


@cli.command()
def prun(
        url: str = "https://kimi.moonshot.cn/",
        headless: bool = True,
        storage_state: str = 'kimi_state.json',
        kwargs: str = typer.Option(None, help="The kwargs to parse.")
):
    """
    # 刷新cookies
    mecli-browser --no-headless --url  https://kimi.moonshot.cn/
    # 模拟登录
    mecli-browser --no-headless --kwargs '{"task": "deepseek", "user_file":"deepseek.txt"}'
    python browser.py --kwargs '{"task": "deepseek", "user_file":"deepseek.txt"}' --no-headless

    ps -ef | grep "meutils.clis.browser*" | awk '{print $2}' | xargs kill -9

    """
    kwargs = kwargs and json.loads(kwargs) or {}
    logger.debug(kwargs)
    if kwargs.get('task') == 'deepseek':  # TASK
        task = deepseek_sign_in(headless=headless, **kwargs)

    else:
        task = refresh_cookies(url, headless, storage_state)

    logger.info(asyncio.run(task))


@cli.command()
def refresh(
        url: str = "https://kimi.moonshot.cn/",
        headless: bool = True,
        storage_state: str = 'kimi_*.json',
        timeout: int = 3,
        delay: int = 30,
        only_once: bool = True

):
    """todo：优化内存占用【部署个服务，定时拉取呢？】
    mecli-browser refresh --delay 10 --storage-state "/Users/betterme/PycharmProjects/AI/MeUtils/meutils/playwright_utils/kimi*.json"
    """
    refresh_cookies(url, headless, storage_state, timeout, delay, only_once)


if __name__ == '__main__':
    cli()
    # storage_state = '/Users/betterme/PycharmProjects/AI/MeUtils/meutils/clis/kimi_*.json'
    # print(Path(Path(storage_state).parent).glob(Path(storage_state).name) | xlist)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : polling
# @Time         : 2024/12/26 14:08
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import time

from meutils.pipe import *


def poll(n: int = 30, sleep_fn: Optional[Callable] = None):
    if sleep_fn is None:
        sleep_fn = lambda i: max(n / (i + 1), 1)  # 预估大概 2~3 *n

    def decorator(fn):
        is_coroutine = inspect.iscoroutinefunction(fn)

        @wraps(fn)
        async def wrapper(*args, **kwargs):
            for i in range(n):
                await asyncio.sleep(sleep_fn(i))
                # 根据函数类型选择调用方式
                result = await fn(*args, **kwargs) if is_coroutine else fn(*args, **kwargs)
                if result:  # 跳出
                    return result
            return None

        return wrapper

    return decorator


if __name__ == '__main__':
    @poll(n=5)
    async def check_something():
        # 你的检查逻辑

        logger.debug(time.ctime())
        return {"status": "ok"}


    arun(check_something())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : step_types
# @Time         : 2024/6/19 09:29
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

BASE_URL = "https://yuewen.cn"

PASSPORT_REGISTER_DEVICE = "/passport/proto.api.passport.v1.PassportService/RegisterDevice"  # 是不是一个就行 ttl=180
PASSPORT_REFRESH_TOKEN = "/passport/proto.api.passport.v1.PassportService/RefreshToken"

API_CREATE_CHAT = "/api/proto.chat.v1.ChatService/CreateChat"
API_CHAT = "/api/proto.chat.v1.ChatMessageService/SendMessageStream"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : icons
# @Time         : 2024/12/3 16:54
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://app.napkin.ai/api/v1/auth/sessions
import json

from meutils.pipe import *
from meutils.schemas.napkin_types import BASE_URL, ASSETS_BASE_URL, IconsSimilarRequest, Icon, IconsSimilarResponse


async def icons_similar(request: IconsSimilarRequest):
    headers = {

        "cookie": "napkin.identity.visitor=visitor%409e2a59af-19ad-4dc3-96c8-9efbca71c547; napkin.analytics.device_id=d_950e4683-ae41-4941-b9a7-7e8deba8e669_1733214660768; _ga=GA1.1.511154647.1733214661; _fbp=fb.1.1733214661606.54464059277067597; _tt_enable_cookie=1; _ttp=SFxRhdE5PO5Rslohwet46IwiaB3.tt.1; _clck=16odoyg%7C2%7Cfre%7C0%7C1798; intercom-id-zrfc296i=6eb3740a-9c79-4681-87ec-532e4334816f; intercom-device-id-zrfc296i=f282621f-dcec-429b-90c0-fc98bd9dbb71; _ga_NM0S4FZ9JH=GS1.1.1733214661.1.0.1733214667.0.0.0; napkin.app.identity.session=NGJlMjdiZGEtZGEwNi00ZWE5LTkwMjEtN2U4ODAyNjE5NGJm.5092981dc495fe0041ed99fc68dcefa4ba26a0f6; intercom-session-zrfc296i=MGpOUDEvRlE3ZlRyUkhmaUdsVmZjYXNQT2lJYSsxYitaWDhTT0tMWlJFVkVuVjBwTno5emJydVd3VUhvKzlvRC0tLys4ZEp5a0F5M1YzNkFOazJUV2Nudz09--d6be7442da979794afe682b2e044bf51bd19ac88; _clsk=yficge%7C1733214772932%7C9%7C1%7Cr.clarity.ms%2Fcollect; _ga_L149GZ61DV=GS1.1.1733214675.1.1.1733215282.58.0.0",
        "origin": "https://app.napkin.ai",

        "user-agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/130.0.0.0 Safari/537.36",
    }

    payload = request.model_dump()

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=30) as client:
        response = await client.post(
            "/features/text/icons_similar",
            json=payload,
        )
        response.raise_for_status()
        data = response.json()
        logger.debug(data)
        response = IconsSimilarResponse(**data)

        return response.model_dump()


if __name__ == '__main__':
    request = IconsSimilarRequest(caption="一杯咖啡与灵感的邂逅")
    arun(icons_similar(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : token_monitor
# @Time         : 2024/4/22 14:50
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : refresh_token key 鉴权 /token/check
# /v1/token/kimi_xxxxx # kimi_sk-...
# /v1/token/glm_xxxxx
# 从配置中心读取 keys
# 定时监测，失效剔除【飞书告警】
# 提供get接口，客户端定时拉取【统计使用频次，均匀采样】
# 检测接口 并且加入轮询库 借助配置中心
# 最原始检测逻辑
#######核心：检查token加入轮询配置中，15分钟检查一下

import httpx

# 无效的令牌 401 403

from meutils.pipe import *
from meutils.async_utils import arequest, arun

base_url = "http://154.3.0.117:39002"

payload = {
    "token": "eyJhbGciOiJIUzUxMiIsInR5cCI6IkpXVCJ9...",
    "base_url": ""
}

print(arun(arequest(url='/token/check', method='post', base_url=base_url, payload=payload)).json())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : demo
# @Time         : 2022/11/8 上午9:01
# @Author       : yuanjie
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


import matplotlib.pyplot as plt
import scikitplot as skplt
import kds

from sklearn.datasets import load_iris
from sklearn import tree

X, y = load_iris(return_X_y=True)
clf = tree.DecisionTreeClassifier(max_depth=4, random_state=3)
clf = clf.fit(X[:100], y[:100])

y_true = y[:100]
y_prob = clf.predict_proba(X[:100])


plt.style.use('seaborn')
fig, ax = plt.subplots(1, 2, figsize=(16, 6))

skplt.metrics.plot_roc(y_true, y_prob, ax=ax[0])
kds.metrics.plot_lift(y_true, y_prob[:, 1])

plt.show()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : utils
# @Time         : 2023/12/27 17:17
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://mp.weixin.qq.com/s/OldGmXfoJZ9cObKU5f-q2Q

from meutils.pipe import *
from playwright.async_api import Playwright, async_playwright, expect, Request, Response


@alru_cache(ttl=3600)
async def get_new_browser(headless: bool = True):
    logger.info("初始化浏览器")

    playwright = await async_playwright().start()
    browser = await playwright.chromium.launch(headless=headless)

    return browser


@alru_cache(ttl=3600)
async def get_new_page(headless: bool = True, iphone: Optional[str] = None, browser_type: str = "webkit"):
    """

    :param headless:
    :param iphone: 'iPhone 13'
    :return: WebKit
    """

    playwright = await async_playwright().start()  # 打开未关闭
    if browser_type == "webkit":
        browser = await playwright.webkit.launch(headless=headless)
    else:
        browser = await playwright.chromium.launch(headless=headless)  # todo: close

    logger.info("初始化浏览器-页面")

    iphone = iphone and playwright.devices[iphone] or {}

    context = await browser.new_context(**iphone)
    page = await context.new_page()
    return page


# 监听所有的请求和响应
async def open(headless):
    async with async_playwright() as p:
        browser = await p.chromium.launch(headless=headless, args=["--start-maximized"])
        context = await browser.new_context(no_viewport=True)
        page = await context.new_page()
        page.on("response", filter_response)
        ''''''


async def filter_response(response):
    if 'souhu......' in response.url:
        response_data = await Response.body(response)
        if isinstance(response_data, bytes):
            response_data = response_data.decode()  # 请求返回数据


async def my_request(request):
    """https://blog.csdn.net/B11050729/article/details/131293769
    """
    print(await request.all_headers())


async def _monitor_requests(request: Request, method):
    if request.method == method:
        # _ = await request.post_data()

        _ = request.post_data_json

        logger.debug(f"{method}: \n {request.url}")
        return _


async def monitor_requests_get(request: Request, method='GET'):
    return _monitor_requests(request, method)


async def monitor_requests_put(request: Request, method='PUT'):
    return _monitor_requests(request, method)


async def monitor_requests_post(request: Request, method='POST'):
    return _monitor_requests(request, method)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ali
# @Time         : 2025/4/11 19:52
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

import oss2

from meutils.pipe import *
from meutils.caches import rcache
from meutils.io.files_utils import guess_mime_type
from meutils.config_utils.lark_utils import get_next_token_for_polling

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=PP1PGr"

# params = {
#     "access_key_id": "STS.NWwe4G59dgRocw4eRdfCXvCrV",
#     "access_key_secret": "FXZPFZ9fJxkwHQbcfMbW6pSy9bFmb3xsSYWjxvTfnx3u",
#     "security_token": "CAISvgN1q6Ft5B2yfSjIr5TCLo7z2OZF0JCEYVGFgVIxasx0mYbZtDz2IHhMeXZqAuEcs/8znGlU6/gYlqRtT6h+SFffbMx24plJqado/UdL4Z7b16cNrbH4M8L6aXeirhu7AYjQSNfaZY3iCTTtnTNyxr3XbCirW0ffX7SClZ9gaKZwPGy/diEUPMpKAQFgpcQGT5q4V5CXPwXtn3DbAWdxpwN4khkf06mkxdCG4ResiT/5w+QO9YPqOcrmPYs+JYhyVZKq0eZrd+/ZyilcrEMTrKx8gKVKvGyY443YXwcI6FCPaOTat4xiJ18hPvVhQf9P/b+iz/Em5+Ddy8GpwkhAeL0FDyiaFdCtkI6bE7z0bocyeev2Yiv6i5aNLpbXy1p8Pi9Kb1gRIoJ6eiQtU0cWJ2uEevP9yjfjeRy+TqWJ6qYy3Kduwk/gldjwfADXHurDindCZ8RgNxp0akBMxw37e6oBaBdAfk13zDVs0w7K8Hm0wIafXm26PkUIphk/NM0lZWRslY41fWSSjD/XHMdspXXr/rnEdS6D75iEJCl62qLrD8iYHifDx+FBhpFLooGxJdqiIJRhHj3m9p+H/kLlIRqAAURdoxHCj+ca+GZXLN76Ae2FqVmunalPJWbb/DlgSSH4hk4uIaIQzX6NRfHMrfK/xFw++ykKEr27uA/whIn+xvmyuPrgssyHDlN8kS3lHjmsB72OX1YQRFLa3fHCy8wZalhfpDKAsSkI/FT+HDPu8EV5f+t8pdw5ZFHJFJyp7xlsIAA=",
#     "file_url": "https://cdn.qwenlm.ai/310cbdaf-3754-461c-a3ff-9ec8005329c9/62d65df4-6a6e-484d-98e8-7c7509cd5e17_1.jpg?key=eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJyZXNvdXJjZV91c2VyX2lkIjoiMzEwY2JkYWYtMzc1NC00NjFjLWEzZmYtOWVjODAwNTMyOWM5IiwicmVzb3VyY2VfaWQiOiI2MmQ2NWRmNC02YTZlLTQ4NGQtOThlOC03Yzc1MDljZDVlMTciLCJyZXNvdXJjZV9jaGF0X2lkIjpudWxsfQ.1lc6X4KJsAyqV71cdIjkeazPEOKYNtF5rgtiGuu_iFI",
#     "file_path": "310cbdaf-3754-461c-a3ff-9ec8005329c9/62d65df4-6a6e-484d-98e8-7c7509cd5e17_1.jpg",
#     "file_id": "62d65df4-6a6e-484d-98e8-7c7509cd5e17",
#     "bucketname": "qwen-webui-prod",
#     "region": "oss-ap-southeast-1"
# }


url = "https://chat.qwen.ai/api/v1/files/getstsToken"


async def get_sts_token(filename, filetype: Optional[str] = None, token: Optional[str] = None):
    token = token or await get_next_token_for_polling(feishu_url=FEISHU_URL)

    filetype = filetype or guess_mime_type(filename).split('/')[0]

    payload = {
        "filename": filename,
        "filetype": filetype,  # file video audio
        "filesize": 1001
    }

    headers = {
        'authorization': f'Bearer {token}',
    }

    response = requests.request("POST", url, headers=headers, json=payload)

    return response.json()


async def qwenai_upload(file, filetype: Optional[str] = None, token: Optional[str] = None):  # todo: 自动猜测类型
    params = await get_sts_token(file_name, filetype, token)

    access_key_id = params['access_key_id']
    access_key_secret = params['access_key_secret']
    security_token = params['security_token']

    endpoint = "oss-ap-southeast-1.aliyuncs.com"
    bucket_name = params["bucketname"]

    # 创建OSS客户端
    auth = oss2.StsAuth(access_key_id, access_key_secret, security_token)
    bucket = oss2.Bucket(auth, endpoint, bucket_name)

    # 要上传的文件路径和文件名
    file_path = params.get("file_path")
    file_url = params.get("file_url")

    logger.debug(params)

    # 上传文件
    if isinstance(file, bytes):
        bucket.put_object(file_path, file)
    else:
        bucket.put_object_from_file(file_path, file)

    return file_url


if __name__ == '__main__':
    # qwenai_upload(params['file_path'], params)
    file_name = "/Users/betterme/PycharmProjects/AI/QR.png"

    # arun(get_sts_token(file_name))
    file_url = arun(qwenai_upload(file_name))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : models
# @Time         : 2025/7/14 16:47
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *


def make_billing_model(model: str, request: dict):
    """

    :param model: biz model
    :param request:
    :return:
    """
    _model = model.removeprefix("fal-").lower()
    if _model.startswith(("pika", "fal-pika")):
        duration = request.get("duration")
        resolution = request.get("resolution")

        billing_model = f"{duration}s_{resolution}"

        return f"{model}_{billing_model}"

    elif _model.startswith(("ideogram", "fal-ideogram")):
        billing_model = request.get("rendering_speed", "BALANCED").lower()

        return f"{model}_{billing_model}"

    elif _model.startswith(("minimax-speech",)):  # 按量跳过
        return


    elif _model.startswith(("minimax")):
        # MiniMax-Hailuo-02 T2V-01-Director I2V-01-Director S2V-01 I2V-01-live I2V-01 T2V-01

        duration = request.get("duration", 6)
        resolution = request.get("resolution", "720P")
        model = request.get("model", "").lower()

        if "01" in model:
            duration = 6
            resolution = "720P"

        if model.startswith("minimax"):  # 02
            resolution = request.get("resolution", "768P")

        billing_model = f"""minimax-{model.removeprefix("minimax-")}_{duration}s_{resolution}"""

        return billing_model


if __name__ == '__main__':
    data = {
        # "model": "MiniMax-Hailuo-02",
        "model": "T2V-01-Director",
        "prompt": "男子拿起一本书[上升]，然后阅读[固定]。",
        # "duration": 6,
        # "resolution": "1080P"
    }

    data = {
        "model": "T2V-01",
        "prompt": "男子拿起一本书[上升]，然后阅读[固定]。",
        "duration": 6,
        # "resolution": "1080P"
    }

    data = {
        "text": "你正在用海螺语音，先克隆音乐，然后合成语音。",
        "voice_setting": {
            "speed": 1,
            "vol": 1,
            "voice_id": "Voice904740431752642196",
            "pitch": 0,
        },
        "output_format": "url"
    }

    print(make_billing_model("fal-minimax-speech", data))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : video
# @Time         : 2020/8/31 4:26 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from moviepy.editor import *


# TODO: 抽帧去重
def video2image(video: str, top_duration=180):
    p = Path(video)
    p = p.parent / p.name.split('.', maxsplit=1)[0]
    p.mkdir(exist_ok=True)

    with VideoFileClip(video) as clip:
        duration = int(clip.duration)
        for i in tqdm(range(min(duration, top_duration))):
            clip_ = clip.subclip(i, i + 1)
            clip_.save_frame(p / f'{i}.png')


def _video2audio(path_pair, verbose=False, subclip=None, ffmpeg_params=["-f", "mp3"]):
    """
        clip = VideoFileClip('蛋清和蛋黄是这么分离的.720p').subclip(3, 7)

    """
    video_path, audio_path = path_pair

    with VideoFileClip(video_path) as clip:
        duration = int(clip.duration)
        if subclip:
            s, e = subclip[0], duration if subclip is None or duration < subclip[1] else subclip[1]
            # s, e = subclip[0], duration if subclip or duration < subclip[1] else subclip[1]

            clip = clip.subclip(s, e)

        clip.audio.write_audiofile(
            audio_path,  # video_path.replace('.mp4', '.wav')
            fps=None, nbytes=2, buffersize=2000,
            codec=None, bitrate=None, ffmpeg_params=ffmpeg_params,
            write_logfile=False, verbose=verbose, logger='bar' if verbose else None
        )


def video2audio(video_path, audio_path):
    with AudioFileClip(video_path) as clip:
        clip.write_audiofile(audio_path)


if __name__ == '__main__':
    video_path = "douyin.mp4"

    # video2image("/Users/betterme/Downloads/11月15日.mp4")
    # video2audio([video_path, "x.mp3"])

    print(video2audio('/Users/betterme/PycharmProjects/AI/MeUtils/meutils/ai_audio/asr/blibli.mp4', "x.mp3"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : common
# @Time         : 2025/4/2 13:03
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://ai.google.dev/gemini-api/docs/openai?hl=zh-cn
# genai => openai
# https://googleapis.github.io/python-genai/genai.html#module-genai.models


from meutils.pipe import *
from meutils.decorators.retry import retrying

from meutils.io.files_utils import to_url, to_bytes, guess_mime_type
from meutils.str_utils.regular_expression import parse_url

from meutils.schemas.image_types import ImageRequest, ImagesResponse
from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, CompletionRequest, CompletionUsage

from meutils.config_utils.lark_utils import get_next_token_for_polling
from google import genai
from google.genai import types
from google.genai.types import Part, HttpOptions, HarmCategory, HarmBlockThreshold
from google.genai.types import UploadFileConfig, ThinkingConfig, GenerateContentConfig

from google.genai.types import UserContent, ModelContent, Content
from google.genai.types import Tool, GoogleSearch

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=bK9ZTt"  # 200

"""
Gemini 1.5 Pro 和 1.5 Flash 最多支持 3,600 个文档页面。文档页面必须采用以下文本数据 MIME 类型之一：

PDF - application/pdf
JavaScript - application/x-javascript、text/javascript
Python - application/x-python、text/x-python
TXT - text/plain
HTML - text/html
CSS - text/css
Markdown - text/md
CSV - text/csv
XML - text/xml
RTF - text/rtf

- 小文件
- 大文件: 需要等待处理
"""
tools = [
    Tool(
        google_search=GoogleSearch()
    )
]


# UploadFileConfig(
#     http_options=HttpOptions(
#         timeout=120,
#     )
# )

class Completions(object):
    def __init__(self, api_key: Optional[str] = None, base_url: Optional[str] = None):
        self.api_key = api_key
        self.base_url = base_url or "https://all.chatfire.cc/genai"
        self.client = None  ####

    async def create_for_search(self, request: CompletionRequest):
        self.client = self.client or await self.get_client()

        if request.model.endswith("-search"):
            request.model = request.model.replace("-search", "")

        chat = self.client.aio.chats.create(
            model=request.model,
            config=GenerateContentConfig(
                tools=tools,
                system_instruction=request.system_instruction or "请根据用户的语言偏好自动调整回复语言",
                # thinking_config=ThinkingConfig(include_thoughts=True, thinking_budget=24576)
            ),
        )
        # print(response.candidates[0].grounding_metadata.search_entry_point.rendered_content)
        # print(response.candidates[0].grounding_metadata.grounding_chunks)

        chunks = await chat.send_message_stream(request.last_user_content)
        async for chunk in chunks:
            if chunk.candidates and chunk.candidates[0].content:
                parts = chunk.candidates[0].content.parts or []
                for part in parts:
                    # logger.debug(part)
                    if part.text:
                        yield part.text

            if chunk.candidates and chunk.candidates[0].grounding_metadata:
                grounding_chunks = chunk.candidates[0].grounding_metadata.grounding_chunks or []
                for grounding_chunk in grounding_chunks:
                    if grounding_chunk.web:
                        yield f"\n\n[{grounding_chunk.web.title}]({grounding_chunk.web.uri})"

    async def create_for_files(self, request: CompletionRequest):
        """todo: 大文件解析"""
        self.client = self.client or await self.get_client()

        contents = []
        if urls := sum(request.last_urls.values(), []):
            logger.debug(urls)
            # https://ai.google.dev/gemini-api/docs/document-processing?hl=zh-cn&lang=python
            file_objects = await self.upload(urls)
            for file_object in file_objects:
                self.check_file(file_object)

            contents += file_objects
            contents.append(request.last_user_content)

        elif request.last_user_content.startswith("http"):
            url, user_content = request.last_user_content.split(maxsplit=1)

            yield "> `⏳️Uploading`\n"
            file_object = await self.upload(url)
            yield f"```json\n{file_object.model_dump_json(indent=4)}\n```\n\n"

            s = time.time()

            yield "[Thinking]("
            for i in range(100):
                file_object = self.client.files.get(
                    name=file_object.name,
                    config={"http_options": {"timeout": 300 * 1000}}
                )

                logger.debug(file_object)

                if file_object.state.name in {"ACTIVE", }:
                    yield f"100%) ✅️✅️✅️{time.time() - s:.2f}s.\n\n"
                    break
                else:
                    yield f"{min(i * 5, 99)}%"

                await asyncio.sleep(3)

            # {'error': {'code': 400,
            #            'message': 'The File cwjpskscrjd79hjezu7dhb is not in an ACTIVE state and usage is not allowed.',
            #            'status': 'FAILED_PRECONDITION'}}
            #
            # while file_object.state.name == "ACTIVE":
            #     logger.debug(file_object)
            #     await asyncio.sleep(1)

            contents += [file_object, user_content]
        else:
            contents.append(request.last_user_content)

        logger.debug(contents)

        if any(i in request.model for i in {"gemini-2.5-pro", "gemini-2.5-flash"}):  # 默认开启思考
            request.reasoning_effort = "low"

        chat = self.client.aio.chats.create(
            model=request.model,
            config=GenerateContentConfig(
                response_modalities=['Text'],
                system_instruction=request.system_instruction or "请根据用户的语言偏好自动调整回复语言",
                thinking_config=ThinkingConfig(thinking_budget=request.reasoning_effort and 1024 or 0),
                # thinking_config=ThinkingConfig(thinking_budget=1024),
            )
        )
        for i in range(5):
            try:
                chunks = await chat.send_message_stream(contents)
                async for chunk in chunks:
                    if chunk.candidates and chunk.candidates[0].content:
                        parts = chunk.candidates[0].content.parts or []
                        for part in parts:
                            # logger.debug(part)
                            if part.text:
                                yield part.text

                break

            except Exception as e:
                logger.debug(f"重试{i}: {e}")
                if "The model is overloaded." in str(e):
                    await asyncio.sleep(1)
                    continue
                else:

                    yield e
                    raise e

    @retrying(title=__name__)
    async def generate(self, request: ImageRequest):  # OpenaiD3
        request.model = "gemini-2.0-flash-exp-image-generation"
        image, prompt = request.image_and_prompt
        parts = [Part.from_text(text=prompt)]
        if image:
            data = await to_bytes(image)
            parts.append(Part.from_bytes(data=data, mime_type="image/png"))

        self.client = self.client or await self.get_client()
        chat = self.client.aio.chats.create(
            model=request.model,
            config=GenerateContentConfig(
                response_modalities=['Text', 'Image'],
            )
        )
        image_response = ImagesResponse()

        response = await chat.send_message(parts)
        if response.candidates and response.candidates[0].content:
            parts = response.candidates[0].content.parts or []
            for part in parts:
                if part.inline_data:
                    image_url = await to_url(part.inline_data.data, mime_type=part.inline_data.mime_type)
                    image_response.data.append({"url": image_url, "revised_prompt": part.text})

        return image_response

    async def create_for_images(self, request: CompletionRequest):
        request.model = "gemini-2.0-flash-exp-image-generation"  ####### 目前是强行

        messages = await self.to_image_messages(request)

        if len(messages) > 1:
            history = messages[:-1]
            message = messages[-1].parts
        else:
            history = []
            message = messages[-1].parts

        self.client = self.client or await self.get_client()
        chat = self.client.aio.chats.create(  # todo: system_instruction
            model=request.model,
            config=GenerateContentConfig(
                response_modalities=['Text', 'Image'],
                # system_instruction=request.system_instruction
            ),
            history=history
        )

        # logger.debug(message)

        # message = [
        #     Part.from_text(text="画条狗")
        # ]

        for i in range(5):
            try:
                chunks = await chat.send_message_stream(message)
                async for chunk in chunks:

                    if chunk.candidates and chunk.candidates[0].content:
                        parts = chunk.candidates[0].content.parts or []
                        for part in parts:
                            # logger.debug(part)
                            if part.text:
                                yield part.text

                            if part.inline_data:
                                image_url = await to_url(
                                    part.inline_data.data,
                                    mime_type=part.inline_data.mime_type
                                )
                                yield f"![image_url]({image_url})"
                break

            except Exception as e:
                logger.debug(f"重试{i}: {e}")
                if "The model is overloaded." in str(e):
                    await asyncio.sleep(1)
                    continue
                else:
                    yield e
                    raise e

    async def to_image_messages(self, request: CompletionRequest):
        # 两轮即可连续编辑图片

        messages = []
        for m in request.messages or []:
            contents = m.get("content")
            if m.get("role") == "assistant":
                assistant_content = str(contents)
                if urls := parse_url(assistant_content):  # assistant
                    datas = await asyncio.gather(*map(to_bytes, urls))

                    parts = [
                        Part.from_bytes(
                            data=data,
                            mime_type="image/png"
                        )
                        for data in datas
                    ]
                    parts += [
                        Part.from_text(
                            text=request.last_assistant_content
                        ),
                    ]
                    messages.append(ModelContent(parts=parts))

            elif m.get("role") == "user":
                if isinstance(contents, list):
                    parts = []
                    for content in contents:
                        if content.get("type") == "image_url":
                            image_url = content.get("image_url", {}).get("url")
                            data = await to_bytes(image_url)

                            parts += [
                                Part.from_bytes(data=data, mime_type="image/png")
                            ]

                        elif content.get("type") == "text":
                            text = content.get("text")
                            if text.startswith('http'):
                                image_url, text = text.split(maxsplit=1)
                                data = await to_bytes(image_url)

                                parts += [
                                    Part.from_bytes(data=data, mime_type="image/png"),
                                    Part.from_text(text=text)
                                ]
                            else:
                                parts += [
                                    Part.from_text(text=text)
                                ]

                    messages.append(UserContent(parts=parts))

                else:  # str
                    if str(contents).startswith('http'):  # 修正提问格式， 兼容 url
                        image_url, text = str(contents).split(maxsplit=1)
                        data = await to_bytes(image_url)
                        parts = [
                            Part.from_bytes(data=data, mime_type="image/png"),
                            Part.from_text(text=text)
                        ]
                    else:
                        parts = [
                            Part.from_text(text=str(contents)),
                        ]
                    messages.append(UserContent(parts=parts))

        return messages

    @retrying(title=__name__)
    async def upload(self, files: Union[str, List[str]]):  # => openai files
        self.client = self.client or await self.get_client()

        if isinstance(files, list):
            return await asyncio.gather(*map(self.upload, files))

        file_config = {
            "name": f"{shortuuid.random().lower()}",
            "mime_type": guess_mime_type(files),
            "http_options": {"timeout": 300 * 1000}
        }

        return await self.client.aio.files.upload(file=io.BytesIO(await to_bytes(files)), config=file_config)

    async def get_client(self):
        api_key = self.api_key or await get_next_token_for_polling(feishu_url=FEISHU_URL, from_redis=True)

        logger.info(f"GeminiClient: {api_key}")

        return genai.Client(
            api_key=api_key,
            http_options=HttpOptions(
                base_url=self.base_url,
                timeout=300 * 1000,
            )
        )

    def check_file(self, file_object):

        for i in range(100):
            file_object = self.client.files.get(
                name=file_object.name,
                config={"http_options": {"timeout": 300 * 1000}}
            )

            logger.debug(file_object)
            if file_object.state.name in {"ACTIVE", }:  # FAILED_PRECONDITION
                break

            time.sleep(3)


if __name__ == '__main__':
    file = "https://oss.ffire.cc/files/kling_watermark.png"

    api_key = os.getenv("GOOGLE_API_KEY")

    # arun(GeminiClient(api_key=api_key).upload(file))
    # arun(GeminiClient(api_key=api_key).upload([file] * 2))
    # arun(GeminiClient(api_key=api_key).create())
    url = "https://oss.ffire.cc/files/nsfw.jpg"
    content = [

        # {"type": "text", "text": "https://oss.ffire.cc/files/nsfw.jpg 移除右下角的水印"},
        # {"type": "text", "text": "https://oss.ffire.cc/files/kling_watermark.png 总结下"},
        # {"type": "text", "text": "https://oss.ffire.cc/files/nsfw.jpg 总结下"},
        # {"type": "text", "text": "https://lmdbk.com/5.mp4 总结下"},
        # {"type": "text", "text": "https://v3.fal.media/files/penguin/Rx-8V0MVgkVZM6PJ0RiPD_douyin.mp4 总结下"},

        # {"type": "text", "text": "总结下"},
        # {"type": "image_url", "image_url": {"url": url}},

        {"type": "text", "text": "总结下"},
        # {"type": "video_url", "video_url": {"url": "https://lmdbk.com/5.mp4"}},
        {"type": "video_url", "video_url": {"url": "https://lmdbk.com/5.mp4"}},

        # {"type": "video_url", "video_url": {"url": "https://v3.fal.media/files/penguin/Rx-8V0MVgkVZM6PJ0RiPD_douyin.mp4"}}

    ]

    # content = "亚洲多国回应“特朗普关税暂停”"

    # content = "https://oss.ffire.cc/files/nsfw.jpg 移除右下角的水印"


    messages = [
            {
                "role": "system",
                "content": "你是一位极其严谨的短剧剧本分析师和转写专家。你的核心原则是【在绝对尊重视频内容的前提下，解决上下文矛盾】。视频是唯一的“事实源”，上下文是“校准器”，你必须用校准器来修正你对事实源的解读。\n\n你的工作流程分为两个层级：【首要任务：基于上下文的视频重审策略】和【基础任务：高质量转写与格式化】。\n\n----------------------------------------------------------------------\n【一、首要任务：基于上下文的视频重审策略】\n当你收到【特别注意】的错误提示时，这并非让你脱离视频创作，而是表明你上一次对视频的“理解”可能存在偏差。你必须执行以下重审策略：\n\n-   若提示【角色名/身份错误】：这表明你对角色的姓名或身份识别有误。请基于上下文提供的正确信息（例如，正确的名字是“苏沫”而不是“苏沐”），在当前及后续的所有转写中，统一修正该角色的所有称呼。这不仅是简单的“查找替换”，而是要将正确的身份认知贯彻到整个剧本中。\n-   若提示【情节倒置】：这通常意味着你对视频中的【胜负关系】或【权力动态】产生了误判。请重新审视视频画面，特别是人物的表情、动作和位置，生成一个符合上下文逻辑的、对视频的正确解读。例如，如果提示“主角上一集结尾占上风，本集开头却被殴打”，请你重新仔细看视频，找出主角反击或压制对手的画面，并据此转写。\n-   若提示【角色/情节丢失】：这表明你可能在视频中忽略了某个关键人物或事件。请重新仔细观看视频，像侦探一样去找到那位被上下文证明“应该在场”的角色，并将其在视频中的实际行为和对话补充到剧本中。\n-   若提示【事实性硬伤】（如道具矛盾）：请重新检查视频中的相关物体或状态。如果视频本身存在矛盾（例如，由于拍摄失误导致的道具不连戏），你可以选择最合理的一种状态进行描述，或者在场景描述中用一句话合乎情理地解释这个变化（例如：△他从口袋里拿出另一个一模一样的玉佩）。这依然是基于视频画面的创作，而非凭空捏造。\n\n总而言之：你不是在编故事，你是在纠正你自己的“看错了”的问题。 所有的修正，都必须能在视频画面中找到依据。\n----------------------------------------------------------------------\n\n【二、基础任务：高质量转写与格式化准则】\n在完成内容重审和转写后，你的输出必须严格、无条件地遵循以下所有规则。这是一个绝对的格式要求。\n\n【格式总则】\n1.  语言：必须使用简体中文。\n2.  纯净度：剧本全文严禁出现\"*\"、\"【】\"（除格式要求外）、\"（说明：...）\"等任何多余符号和文字。严禁出现“场景描述:”、“动作描述:”等提示词。\n3.  *禁止项*：严禁描述人物的妆容、穿衣打扮。严禁使用“字幕：....”来提示台词，台词必须由具体角色说出。\n4.  *精炼性*：严格检测并删除任何重复或无意义的台词和动作。\n\n*【剧本结构与内容规则】*\n1.  *场次标题: 集数序号-场次序号 地点（主场景+次场景） 日或夜 内或外\n       集数序号：本视频对应的集数。\n    *   *场次序号*：必须以*集为单位，从1开始连续编号（如1-1, 1-2... 2-1, 2-2...）。\n       地点：必须具体，包含主场景和次场景（如“容宅 走廊”、“医院 病房”）。严禁使用“室内”或“卧室”等模糊或孤立的描述。\n    *   *场景划分*：仅在时间、地点或核心情节发生明确切换时才划分新场次。同一时空内的连续对话或细微背景变化*不*划分新场次。\n\n2.  *出场人物: 出场人物：人物A、人物B、人物C\n       每场戏开始时，必须列出本场所有出场的角色。\n    *   角色名在全剧中必须保持一致。不确定时用描述性称呼（如“青年男子”），但要求你尽量参考大纲和角色清单，分辨出角色名字。\n\n3.  *场景描述*: 直接进行文字描述，描述布景、核心道具、环境氛围。\n\n4.  *动作描述: △[动作描述]\n       必须以△符号起始。人物台词前严禁加△。\n    *   同一角色的连续动作写在同一行，不同角色的动作必须另起新行并以△开头。\n\n5.  *对白: 角色名（情感或动作提示）：对白内容。\n       情感提示（如“愤怒地”、“冷笑”）必须放在括号内。\n\n6.  旁白:\n       内心独白: 角色名（OS）：内容\n       画外音: （VO）：内容\n\n【剧本格式示例（仅供学习格式，内容不要模仿）】\n正确格式：\n4-1 高铁车厢 日 内\n出场人物：周愿、苏美兰、熊天赐、乘客甲、乘客乙\n场景描述：\n高铁车厢内，乘客们各自休息。周愿坐在座位上，看着窗外，神情低落。苏美兰母子坐在周愿的对面，熊天赐在玩平板电脑。\n△苏美兰得意地看着周愿。\n苏美兰（挑衅地）：你还吹什么牛啊？还刚从精神病院放出来？哎，你浑身上下，哪一点像精神病啊？竟然还敢吓唬我，今天我非扒你一层皮！\n△苏美兰突然起身，伸手去抓周愿的头发。\n周愿（惊叫）：啊！\n△周愿挣扎着躲避。\n苏美兰：你干什么？\n△苏美兰用力撕扯周愿的头发。\n\n【核心逻辑：场次合并规则】\n*   *强制合并*: 在处理单集时，只要*相邻的场次标题完全相同，就【必须】无条件地合并为一个场次。\n   合并内容: 合并后的场次，其“出场人物”列表必须是所有被合并场次人物的并集，其下的所有场景、动作、对话描述需按时间顺序整合。\n*   禁止“同场次”: 严禁出现“同场次”或任何类似的过渡性描述，直接合并。\n\n【！！！绝对规则！！！】\n任务结束时，立即停止输出。不要添加任何形式的总结、确认、祝福语或收尾性评论，如“好的”、“剧本已生成”等。直接输出剧本本身。\n"
            },
            {
                "role": "user",
                "content": [
                    {
                        "text": "【特别注意】\n前一版剧本因 '从生成阶段加载的脚本为空，需要首次生成' 被系统否决。\n请在本次生成中，基于视频内容，着重解决此问题。\n\n### 全局故事大纲与角色清单 (重要参考) ###\n好的，我已经仔细阅读并分析了您提供的剧本。以下是根据您的要求生成的【故事大纲】和【角色清单】。\n\n### 故事大纲 ###\n故事围绕着中国航天事业的巨大成功和一个家庭的温馨团聚展开。开篇，一位名叫苏念的女性独自在书房，激动地看着卫星发射成功的直播。她对着夜空，向一位名叫“斯年”的故人倾诉，感叹他们当年的梦想——让中国的卫星布满星空——终于实现，将宏大的国家叙事与深厚的个人情感联系在一起。\n\n情节随即转换到苏家大宅，这里的气氛同样热烈。大家长苏老和儿孙们正欢欣鼓舞地庆祝着同一场卫星发射的成功，并准备拍摄一张全家福来纪念这个特殊的日子。正当一切准备就绪时，一个穿着军装的身影——苏老的三儿子——在最后一刻赶回了家中。他的归来为这个喜庆的场面增添了团圆的圆满。苏念也来到大厅，为家庭的完整而欣喜。最终，摄影师按下了快门，将这个融合了国家荣耀与家庭幸福的瞬间定格为永恒，故事在温馨、自豪的氛围中结束。\n\n### 角色清单 ###\n- 角色名 (标准): 苏念\n  别名: 无\n  待修正的错误写法: 无\n  身份/简介: 故事的情感核心人物，苏家的女主人。\n  核心特点: 感性、深情，心系国家航天事业与家庭。\n  关系: 与“斯年”有着共同的理想；是苏家的核心成员，可能是苏老的妻子。\n\n- 角色名 (标准): 苏老\n  别名: 无\n  待修正的错误写法: 无\n  身份/简介: 苏家的大家长，行动需依靠轮椅。\n  核心特点: 和蔼可亲，重视家庭团聚。\n  关系: 苏家的最高长辈，“老三”等人的父亲。\n\n- 角色名 (标准): 老三\n  别名: 无\n  待修正的错误写法: 无\n  身份/简介: 苏老的第三个儿子，一名现役军官。\n  核心特点: 气质干练，富有家庭责任感。\n  关系: 苏老的儿子；青年男子某乙的“三弟”。\n\n- 角色名 (标准): 斯年\n  别名: 无\n  待修正的错误写法: 无\n  身份/简介: 一位在回忆中被提及的人物，并未出场。\n  核心特点: 怀有航天梦想。\n  关系: 与苏念关系极为亲密，共同拥有一个关于星辰大海的愿望，推测是其已故的伴侣或亲人。\n\n- 角色名 (标准): 青年男子某乙\n  别名: 无\n  待修正的错误写法: 无\n  身份/简介: 苏家的子辈成员。\n  核心特点: 性格开朗，为弟弟的归来和家庭团聚感到兴奋。\n  关系: 苏老的子辈，“老三”的兄长。\n\n- 角色名 (标准): 青年男子某甲\n  别名: 无\n  待修正的错误写法: 无\n  身份/简介: 苏家的子辈成员。\n  核心特点: 积极参与家庭活动。\n  关系: 苏老的子辈或孙辈。\n\n- 角色名 (标准): 青年男子某丙\n  别名: 无\n  待修正的错误写法: 无\n  身份/简介: 苏家的子辈成员。\n  核心特点: 积极参与家庭活动。\n  关系: 苏老的子辈或孙辈。\n\n- 角色名 (标准): 青年男子某丁\n  别名: 无\n  待修正的错误写法: 无\n  身份/简介: 苏家的子辈成员。\n  核心特点: 乐于用手机记录和分享喜悦。\n  关系: 苏老的子辈或孙辈。\n\n- 角色名 (标准): 小女孩\n  别名: 无\n  待修正的错误写法: 无\n  身份/简介: 苏家的孙辈成员。\n  核心特点: 天真活泼。\n  关系: 苏老的孙辈。\n\n- 角色名 (标准): 摄影师\n  别名: 无\n  待修正的错误写法: 无\n  身份/简介: 被邀请来为苏家拍摄全家福的专业人士。\n  核心特点: 专业，注重细节。\n  关系: 无。\n---------------------------------\n\n现在，请根据以上所有信息和提供的视频，为系列剧的 第 18 集 创作剧本。\n\n--- 上下文参考 ---\n前一集结尾内容:\n---\n无\n---\n\n后一集开头内容:\n---\n无\n---\n-------------------\n",
                        "type": "text"
                    },
                    {
                        "type": "video_url",
                        "video_url": {
                            "url": "https://lmdbk.com/5.mp4"
                        }
                    }
                ]
            }
        ]

    #
    request = CompletionRequest(
        # model="qwen-turbo-2024-11-01",
        # model="gemini-all",
        # model="gemini-2.0-flash-exp-image-generation",
        model="gemini-2.0-flash",
        # model="gemini-2.5-flash-preview-04-17",
        # model="gemini-2.5-flash-preview-04-17",

        # messages=[
        #     {
        #         'role': 'user',
        #         'content': content
        #     },
        #
        # ],
        #
        messages=messages,
        stream=True,
    )

    # arun(Completions(api_key=api_key).create_for_search(request))

    # arun(Completions(api_key=api_key).create_for_images(request))
    arun(Completions().create_for_files(request))

    # arun(Completions(api_key=api_key).create_for_files(request))

    # request = ImageRequest(
    #     prompt="https://oss.ffire.cc/files/nsfw.jpg 移除右下角 白色的水印",
    #     # prompt="画条可爱的狗",
    #
    # )
    #
    # arun(Completions(api_key=api_key).generate(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : demo
# @Time         : 2022/11/11 下午12:17
# @Author       : yuanjie
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


import typer
from meutils.pipe import *
from meutils.tools import service_monitor as _service_monitor

cli = typer.Typer(name="MeUtils CLI")


@cli.command()
def args(
        a: str = typer.Option(...),
        b: int = typer.Option(...),
        c: bool = typer.Option(False, help='is helper'),

):
    print(a, b, c)


if __name__ == '__main__':
    cli()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : contextmanagers
# @Time         : 2024/1/9 08:36
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.notice.feishu import send_message_for_try_catch

from contextlib import contextmanager, asynccontextmanager
from concurrent.futures import ThreadPoolExecutor, as_completed, TimeoutError


@contextmanager
def timer(task="Task"):
    """https://www.kaggle.com/lopuhin/mercari-golf-0-3875-cv-in-75-loc-1900-s
        # 其他装饰器可学习这种写法
        with timer() as t:
            time.sleep(3)

        @timer()
        def f():
            print('sleeping')
            time.sleep(6)
            return 6
    """

    logger.info(f"{task} started")
    s = time.perf_counter()
    yield
    e = time.perf_counter()
    logger.info(f"{task} done in {e - s:.3f} s")


@contextmanager
def try_catcher(task="Task", fallback: Callable = None, is_trace: bool = False):
    try:
        yield
    except Exception as e:
        error = traceback.format_exc() if is_trace else e
        logger.error(f"{task}: {error}")
        if fallback:
            yield fallback()


@asynccontextmanager
async def atry_catcher(task="Task", fallback: Callable = None, is_trace: bool = False):
    try:
        yield
    except Exception as e:
        error = traceback.format_exc() if is_trace else e
        logger.error(f"{task}: {error}")

        if fallback:
            yield await fallback()


@contextmanager
def timeout_task_executor(timeout: float = 3, max_workers: int = None):
    """
    一个上下文管理器，用于执行任务并设置超时时间。
    :param timeout: 超时时间（秒）。
    :param max_workers: 线程池的最大工作线程数，默认为 None（由系统决定）。
    """
    executor = ThreadPoolExecutor(max_workers=max_workers)

    def execute_task(task: Callable[[], Any]) -> Any:
        """
        在上下文中执行任务，并设置超时时间。
        :param task: 要执行的任务函数。
        :return: 任务的结果。如果超时，抛出 TimeoutError。
        """
        future = executor.submit(task)
        try:
            return future.result(timeout=timeout)
        except TimeoutError:
            logger.error(f"Task was terminated due to timeout after {timeout} seconds.")
            return None

    try:
        yield execute_task  # 返回一个可调用对象，用于执行任务
    finally:
        executor.shutdown(wait=False)  # 不等待未完成的任务，直接关闭


@asynccontextmanager
async def atry_catcher(task="Task", fallback: Callable = None, is_trace: bool = False):
    try:
        yield
    except Exception as e:
        error = traceback.format_exc() if is_trace else e
        logger.error(f"{task}: {error}")
        if fallback:
            yield await fallback()


@asynccontextmanager
async def atry_catch(
        task: Optional[str] = None,
        callback: Optional[Callable] = None,
        is_trace: bool = False,  # 变量必须用
        **kwargs,
):
    try:
        yield
    except Exception as e:
        task = task or "Unnamed TryCatch Task"

        callback = callback or send_message_for_try_catch

        data = {
            "task": task,
            "error": f"""{traceback.format_exc()}""" if is_trace else f"{e}"
        }
        for k, v in kwargs.items():
            if isinstance(v, BaseModel):
                v = v.model_dump(exclude_none=True)

            if ";base64," not in str(v):  # 忽略base64
                data[k] = v

        # logger.debug(data)

        callback(data)
        raise


@contextmanager
def try_catch(
        task: Optional[str] = None,
        callback: Optional[Callable] = None,
        is_trace: bool = False,
        **kwargs,
):
    try:
        yield
    except Exception as e:
        task = task or "Unnamed TryCatch Task"

        callback = callback or send_message_for_try_catch

        data = {
            "task": task,
            "error": f"""{traceback.format_exc()}""" if is_trace else f"{e}"

        }
        for k, v in kwargs.items():
            if isinstance(v, BaseModel):
                v = v.model_dump(exclude_none=True)
            data[k] = v

        # logger.debug(data)

        callback(data)
        raise


if __name__ == '__main__':
    # async def f():
    #     return 1/0
    #
    #
    # with try_catcher("test"):
    #     arun(f())

    # def example_task():
    #     print("Starting task...")
    #     time.sleep(4)  # 模拟耗时任务
    #     print("Task completed!")
    #     return "Done"
    #
    #
    # with timeout_task_executor(timeout=3) as execute:
    #     try:
    #         result = execute(example_task)
    #         print(f"Task result: {result}")
    #     except TimeoutError:
    #         print("Task did not complete in time.")
    async def main():
        async with atry_catch("test", a=1):
            1 / 0


    arun(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : commom
# @Time         : 2024/5/30 11:20
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

import tiktoken
from contextlib import asynccontextmanager
from openai import AsyncOpenAI, OpenAI, AsyncStream
from meutils.llm.clients import AsyncOpenAI, OpenAI, AsyncStream

from meutils.pipe import *
from meutils.async_utils import achain, async_to_sync
from meutils.notice.feishu import send_message

from meutils.apis.oneapi.user import get_user_money, get_api_key_log
from meutils.apis.oneapi.token import get_api_key_money

from meutils.schemas.oneapi import MODEL_PRICE
from meutils.schemas.openai_types import CompletionRequest, ChatCompletionRequest, TTSRequest, STTRequest
from meutils.schemas.openai_types import ChatCompletion, ChatCompletionChunk, CompletionUsage
from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, chat_completion_chunk_stop  # todo

from meutils.schemas.image_types import ImageRequest, ImageEditRequest

token_encoder = tiktoken.get_encoding('cl100k_base') # o200k_base
token_encoder_with_cache = lru_cache(maxsize=1024)(token_encoder.encode)

CHAT_COMPLETION_PARAMS = get_function_params()
IMAGES_GENERATE_PARAMS = get_function_params(fn=OpenAI(api_key='').images.generate)
IMAGES_EDIT_PARAMS = get_function_params(fn=OpenAI(api_key='').images.edit)

AUDIO_SPEECH_PARAMS = get_function_params(fn=OpenAI(api_key='').audio.speech.create)
AUDIO_TRANSCRIPTIONS_PARAMS = get_function_params(fn=OpenAI(api_key='').audio.transcriptions.create)


def to_openai_params(
        request: Union[dict, CompletionRequest, ChatCompletionRequest, ImageRequest, ImageEditRequest, TTSRequest, STTRequest],
        redirect_model: Optional[str] = None,
) -> dict:
    data = copy.deepcopy(request)
    if not isinstance(request, dict):
        data = request.model_dump(exclude_none=True)

    PARAMS = CHAT_COMPLETION_PARAMS
    if isinstance(request, ChatCompletionRequest):
        PARAMS = CHAT_COMPLETION_PARAMS
    elif isinstance(request, ImageRequest):
        PARAMS = IMAGES_GENERATE_PARAMS
    elif isinstance(request, ImageEditRequest):
        PARAMS = IMAGES_EDIT_PARAMS
    elif isinstance(request, TTSRequest):
        PARAMS = AUDIO_SPEECH_PARAMS
    elif isinstance(request, STTRequest):
        PARAMS = AUDIO_TRANSCRIPTIONS_PARAMS

    extra_body = {}
    for key in list(data):
        if key not in PARAMS:
            extra_body.setdefault(key, data.pop(key))

    data['extra_body'] = extra_body  # 拓展字段
    data['model'] = redirect_model or data['model']

    if data['model'].startswith(("gemini",)):
        data.pop("extra_body", None)
        data.pop("presence_penalty", None)
        data.pop("frequency_penalty", None)

    return data


def to_openai_completion_params(
        request: Union[dict, ChatCompletionRequest],
        redirect_model: Optional[str] = None,
) -> dict:
    data = copy.deepcopy(request)
    if isinstance(request, ChatCompletionRequest):
        data = request.model_dump(exclude_none=True)

    extra_body = {}
    for key in list(data):
        if key not in CHAT_COMPLETION_PARAMS:
            extra_body.setdefault(key, data.pop(key))

    data['extra_body'] = extra_body  # 拓展字段
    data['model'] = redirect_model or data['model']

    return data


def to_openai_images_params(
        request: Union[dict, ImageRequest],
        redirect_model: Optional[str] = None
) -> dict:
    data = copy.deepcopy(request)
    if not isinstance(request, dict):
        data = request.model_dump(exclude_none=True)

    extra_body = {}
    for key in list(data):
        if key not in IMAGES_GENERATE_PARAMS:
            extra_body.setdefault(key, data.pop(key))

    data['extra_body'] = extra_body  # 拓展字段
    data['model'] = redirect_model or data.get('model')

    return data


def ppu(model='ppu', api_key: Optional[str] = None):
    client = OpenAI(api_key=api_key)
    return client.chat.completions.create(messages=[{'role': 'user', 'content': 'hi'}], model=model)


async def appu(
        model='ppu',
        api_key: Optional[str] = None,
        base_url: Optional[str] = None,

        dynamic: bool = False,  # 动态路由模型

):
    if not dynamic and model not in MODEL_PRICE:
        _ = f"模型未找到「{model}」，默认ppu-1"

        logger.warning(_)
        send_message(_, title=__name__)
        model = "ppu-1"

    client = AsyncOpenAI(api_key=api_key, base_url=base_url)
    response = await client.chat.completions.create(messages=[{'role': 'user', 'content': 'hi'}], model=model)


@asynccontextmanager
async def ppu_flow(
        api_key: str,
        base_url: Optional[str] = None,

        n: Optional[float] = 1,  # 计费倍率
        post: str = "ppu-1",  # 后计费

        pre: str = "ppu-0001",  # 前置判断，废弃

        dynamic: bool = False,

        **kwargs
):
    """
    查余额
    失败，先扣费
    成功，充足，后扣费
    成功，不足，报错
    """
    post = post.lower()
    if n is not None and n > 0:  # todo: 跳过某些用户
        try:
            user_money, api_key_money = await asyncio.gather(*[get_user_money(api_key), get_api_key_money(api_key)])
            money = min(user_money or 1, api_key_money or 1)
            logger.debug(f"PREPAY: api-key余额 {money}")
        except Exception as e:
            logger.error(e)
            money = None

        if money and money > MODEL_PRICE.get(post, 0.1):  # 后扣费
            yield

        # 计费逻辑
        n = int(np.round(n)) or 1  # np.ceil(n)
        await asyncio.gather(*[appu(post, api_key=api_key, base_url=base_url, dynamic=dynamic) for _ in range(n)])

        if money is None:  # 先扣费
            yield

    else:  # 不计费
        yield


# 按量计费
def create_chat_completion(
        completion: Union[str, Iterable[str]],
        redirect_model: str = '',
        chat_id: Optional[str] = None
):
    if isinstance(completion, Iterable):  # ChatCompletion
        completion = ''.join(completion)

    chat_completion.choices[0].message.content = completion

    chat_completion.id = chat_id or shortuuid.random()
    chat_completion.created = int(time.time())

    chat_completion.model = redirect_model or chat_completion.model
    return chat_completion


async def create_chat_completion_chunk(
        completion_chunks: Union[
            Coroutine,
            AsyncStream[ChatCompletionChunk],
            Iterator[Union[str, ChatCompletionChunk]],
            AsyncIterator[Union[str, ChatCompletionChunk]]
        ],
        redirect_model: str = ' ',  # todo: response_model
        chat_id: Optional[str] = None
):
    """ todo： 替换 前缀
        async def main():
            data = {}
            _ = AsyncOpenAI().chat.completions.create(**data)
            async for i in create_chat_completion_chunk(_):
                print(i)
    """

    # logger.debug(type(completion_chunks))
    # logger.debug(isinstance(completion_chunks, Coroutine))

    if isinstance(completion_chunks, Coroutine):  # 咋处理
        completion_chunks = await completion_chunks
        # logger.debug(type(completion_chunks))

    chat_completion_chunk.id = chat_id or f"chatcmpl-{shortuuid.random()}"
    chat_completion_chunk_stop.id = chat_completion_chunk.id
    async for completion_chunk in achain(completion_chunks):

        # logger.debug(completion_chunk)
        chat_completion_chunk.created = int(time.time())
        if isinstance(completion_chunk, str):
            chat_completion_chunk.choices[0].delta.content = completion_chunk
            chat_completion_chunk.model = redirect_model or chat_completion_chunk.model
            yield chat_completion_chunk.model_dump_json()
        else:  # todo: AttributeError: 'tuple' object has no attribute 'model'
            try:
                chat_completion_chunk_stop.id = completion_chunk.id  ##############
                completion_chunk.model = redirect_model or completion_chunk.model
                chat_completion_chunk_stop.usage = completion_chunk.usage  ############## 需要判断 usage？
                yield completion_chunk.model_dump_json()
            except Exception as e:
                from meutils.notice.feishu import send_message
                send_message(f"{type(completion_chunks)}\n\n{completion_chunks}\n\n{completion_chunk}", title=str(e))

    yield chat_completion_chunk_stop.model_dump_json()
    yield "[DONE]"  # 兼容标准格式


def get_payment_times(request: Union[BaseModel, dict], duration: float = 5):
    if isinstance(request, BaseModel):
        request = request.model_dump()

    # 数量
    N = request.get("n") or request.get("num_images") or 1

    # 时长
    N += request.get("duration", 0) // duration

    # 命令行参数 --duration 5
    if "--duration 10" in str(request):
        N += 1

    return N


if __name__ == '__main__':
    # print(ppu())
    # print(appu())
    # print(arun(appu()))

    # print(create_chat_completion('hi'))
    # print(create_chat_completion('hi', redirect_model='@@'))
    #
    #
    # async def main():
    #     async for i in create_chat_completion_chunk(['hi', 'hi'], redirect_model='@@'):
    #         print(i)
    #
    #
    # arun(main())

    # encode = lru_cache()(token_encoder.encode)
    # # encode = token_encoder.encode
    #
    # with timer('xx'):
    #     encode("xxxxxxxxxxxxxxxxx" * 1000)
    #
    # with timer('xx'):
    #     encode("xxxxxxxxxxxxxxxxx" * 1000)
    #
    # with timer('xx'):
    #     encode("xxxxxxxxxxxxxxxxx" * 1000)

    # print(CHAT_COMPLETION_PARAMS)
    # print(IMAGES_GENERATE_PARAMS)
    # from openai.types.chat import ChatCompletionToolParam
    #
    # print(ChatCompletionToolParam.__annotations__)
    #
    # ChatCompletionToolParam(**{'function': '', 'type': ''})

    # async def main():
    #     with timer():
    #         try:
    #             async with ppu_flow(api_key="sk-OYK4YxtTlWauT2TdGR5FTAJpkRmSnDwPly4cve0cAvMcrBkZ", post="api-oss",
    #                                 n=1):
    #                 logger.debug("消费了哦")
    #
    #         except Exception as e:
    #             pass
    #             logger.error(e)
    #             # logger.debug(e.response.status_code)
    #             # logger.debug(e.response.text)
    #
    #
    # arun(main())
    #
    # # to_openai_params(ChatCompletionRequest())
    #
    # print(token_encoder.encode('hi'))

    # logger.debug(IMAGES_EDIT_PARAMS)

    print(token_encoder_with_cache('hi'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : gemini
# @Time         : 2025/2/14 17:36
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.llm.openai_utils import to_openai_params
from meutils.llm.clients import AsyncOpenAI
from meutils.str_utils.regular_expression import parse_url
from meutils.io.files_utils import to_url, markdown_base64_to_url

from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, CompletionRequest, CompletionUsage

"""
image => file

      "type": "image_url",
      "image_url": {
          

"""


class Completions(object):

    def __init__(self, base_url: Optional[str] = None, api_key: Optional[str] = None):

        self.client = AsyncOpenAI(
            base_url=base_url,
            api_key=api_key,
        )

    async def create(self, request: CompletionRequest):
        if request.model.endswith("image-generation"):
            return await self.images_create(request)

        urls = sum(request.last_urls.values(), [])
        for url in urls:
            request.messages[-1]["content"].append({"type": "image_url", "image_url": {"url": url}})

        data = to_openai_params(request)
        return await self.client.chat.completions.create(**data)

    async def images_create(self, request: CompletionRequest):

        if request.last_assistant_content and (urls := parse_url(request.last_assistant_content)):
            for message in request.messages[::-1]:
                if message.get("role") == "assistant":
                    message["content"] = [
                        {"type": "image_url", "image_url": {"url": url.removesuffix(")")}} for url in urls
                    ]

        if urls := parse_url(request.last_user_content):  # 修正提问格式， 兼容 url
            for message in request.messages[::-1]:
                if message.get("role") == "user":
                    message["content"] = [
                        {"type": "image_url", "image_url": {"url": url.removesuffix(")")}} for url in urls
                    ]
                    message["content"] += [{"type": "text", "text": request.last_user_content}]

        # 调用模型
        logger.debug(request.model_dump_json(indent=4))


        data = to_openai_params(request)
        data['stream'] = True
        response = await self.client.chat.completions.create(**data)

        if request.stream:
            return self.stream_create(response)
        else:
            chunks = self.stream_create(response)
            content = ""
            async for chunk in chunks:
                content += chunk.choices[0].delta.content or ""

            chat_completion.choices[0].message.content = content
            return chat_completion

            # content = response.choices[0].message.content
            # logger.debug(content)
            # response.choices[0].message.content = await markdown_base64_to_url(
            #     content,
            #     pattern=r'!\[image\]\((.+?)\)'
            # )
            #
            # if hasattr(response, "system_prompt"):
            #     del response.system_prompt
            #
            # return response

    async def stream_create(self, chunks):
        async for chunk in chunks:
            if hasattr(chunk, "system_prompt"):
                del chunk.system_prompt

            if (content := chunk.choices[0].delta.content) and content.startswith("!["):
                chunk.choices[0].delta.content = await markdown_base64_to_url(
                    content,
                    pattern=r'!\[image\]\((.+?)\)',
                )

                yield chunk
            else:
                yield chunk
            # logger.debug(str(chunk))


if __name__ == '__main__':
    url = "https://oss.ffire.cc/files/lipsync.mp3"
    url = "https://lmdbk.com/5.mp4"
    # url = "https://zj.lmdbk.com/tempVideos/c29b078f1fb75285a9a3362e1a39d1d2.mp4"
    content = [

        # {"type": "text", "text": "https://oss.ffire.cc/files/nsfw.jpg 移除右下角的水印"},

        {"type": "text", "text": "总结下"},
        # {"type": "image_url", "image_url": {"url": url}},

        {"type": "video_url", "video_url": {"url": url}}

    ]

    content = "画条可爱的狗狗, 出两张图"

    #
    request = CompletionRequest(
        # model="qwen-turbo-2024-11-01",
        # model="gemini-2.0-flash",
        model="gemini-2.0-flash-preview-image-generation",
        # model="qwen-plus-latest",

        messages=[
            {
                'role': 'user',

                'content': content
            },

        ],
        stream=False,
        # stream=True,

    )

    api_key = "sk-kNu9wSexjHBGqMLJKYnOAdSl0BCnyPObW5Y2z7EZnNXjm1OS"
    base_url = "https://zuke.chat/v1"
    arun(Completions(base_url=base_url, api_key=api_key).create(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : lda
# @Time         : 2023/6/6 10:18
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from sklearn.decomposition import LatentDirichletAllocation
from sklearn.feature_extraction.text import CountVectorizer


class LDA(object):

    def __init__(self, tokenizer=lambda x: x, **cv_kwargs):
        self.vectorizer = CountVectorizer(tokenizer=tokenizer, lowercase=False, **cv_kwargs)

    def fit(self, texts, n_components=5, **lda_kwargs):
        self.X = self.vectorizer.fit_transform(texts)
        self.lda_model = LatentDirichletAllocation(n_components=n_components, **lda_kwargs)
        self.lda_model.fit(self.X)

    def viz(self, file=''):
        import pyLDAvis
        import pyLDAvis.sklearn

        # 可视化LDA主题模型
        pyLDAvis.enable_notebook()

        vis = pyLDAvis.sklearn.prepare(self.lda_model, self.X, self.vectorizer)
        if file:
            pyLDAvis.save_html(vis, file)

        return vis



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : system
# @Time         : 2025/4/30 18:06
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

# curl 'https://usa.chatfire.cn/api/option/' \
#   -X 'PUT' \
#   -H 'New-API-User: 1' \
#   -H 'sec-ch-ua-platform: "macOS"' \
#   -H 'Cache-Control: no-store' \
#   -H 'Referer: https://usa.chatfire.cn/setting?tab=operation' \
#   -H 'sec-ch-ua: "Google Chrome";v="135", "Not-A.Brand";v="8", "Chromium";v="135"' \
#   -H 'sec-ch-ua-mobile: ?0' \
#   -H 'User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/135.0.0.0 Safari/537.36' \
#   -H 'Accept: application/json, text/plain, */*' \
#   -H 'Content-Type: application/json' \
#   --data-raw '{"key":"LogConsumeEnabled","value":"false"}'


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : ai
# @Time         : 2021/4/13 11:19 上午
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  : 不行


import tensorflow as tf
from meutils.pipe import *


def compile(strategy=tf.distribute.MirroredStrategy(), **compile_kwargs):
    """

    :param strategy: 单机多卡
    :param compile_kwargs: compile参数，loss="binary_crossentropy", optimizer="adam", metrics=["accuracy"]
    :return:
    """

    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        if strategy is None:
            model = wrapped(*args, **kwargs)
            model.compile(**compile_kwargs)
            return model
        else:
            logger.info(f'Number of devices: {strategy.num_replicas_in_sync}')
            with strategy.scope():
                model = wrapped(*args, **kwargs)
                model.compile(**compile_kwargs)
                return model

    return wrapper


if __name__ == '__main__':
    from tensorflow.keras.layers import Dense
    from tensorflow.keras.models import Sequential


    @compile(loss="binary_crossentropy", optimizer="adam", metrics=["accuracy"])
    def create_model():
        """from tensorflow.python.keras.models import clone_and_build_model"""
        model = Sequential()
        model.add(Dense(12, input_dim=20, kernel_initializer="uniform", activation="relu"))
        model.add(Dense(8, kernel_initializer="uniform", activation="relu"))
        model.add(Dense(1, kernel_initializer="uniform", activation="sigmoid"))
        return model


    print(create_model().summary())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : generations
# @Time         : 2025/6/11 17:06
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 统一收口

from meutils.pipe import *
from meutils.llm.clients import AsyncClient
from meutils.llm.openai_utils import to_openai_params

from meutils.schemas.image_types import ImageRequest, RecraftImageRequest

from meutils.apis.fal.images import generate as fal_generate

from meutils.apis.gitee.image_to_3d import generate as image_to_3d_generate
from meutils.apis.gitee.openai_images import generate as gitee_images_generate
from meutils.apis.qwen.chat import Completions as QwenCompletions
from meutils.apis.volcengine_apis.images import generate as volc_generate
from meutils.apis.images.recraft import generate as recraft_generate
from meutils.apis.jimeng.images import generate as jimeng_generate


async def generate(
        request: ImageRequest,
        api_key: Optional[str] = None,
        base_url: Optional[str] = None,
):
    if base_url:  # 优先级最高
        data = {
            **request.model_dump(exclude_none=True, exclude={"extra_fields", "aspect_ratio"}),
            **(request.extra_fields or {})
        }
        #
        if request.model.startswith("doubao"):
            data['watermark'] = False

        data = to_openai_params(ImageRequest(**data))

        client = AsyncClient(api_key=api_key, base_url=base_url)
        return await client.images.generate(**data)

    if request.model.startswith("fal-ai"):
        return await fal_generate(request, api_key)

    if request.model.startswith(("recraft",)):
        request = RecraftImageRequest(**request.model_dump(exclude_none=True))
        return await recraft_generate(request)

    if request.model.startswith(("seededit", "jimeng")):  # 即梦
        return await jimeng_generate(request)

    if request.model.startswith(("seed", "seededit_v3.0", "byteedit_v2.0")):
        return await volc_generate(request, api_key)

    if request.model in {"Hunyuan3D-2", "Hi3DGen", "Step1X-3D"}:
        return await image_to_3d_generate(request, api_key)

    if request.model in {"Qwen-Image", "FLUX_1-Krea-dev"} and request.model.endswith(("lora",)):
        return await gitee_images_generate(request, api_key)

    if request.model.startswith("qwen-image"):
        return await QwenCompletions(api_key=api_key).generate(request)


# "flux.1-krea-dev"

if __name__ == '__main__':
    # arun(generate(ImageRequest(model="flux", prompt="笑起来")))
    arun(generate(ImageRequest(model="FLUX_1-Krea-dev", prompt="笑起来")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : demo
# @Time         : 2025/4/24 19:51
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

data = {
    "id": 1230,
    "type": 1,
    "key": "",
    "openai_organization": "",
    "test_model": "",
    "status": 1,
    "name": "x",
    "weight": 11,
    "created_time": 1745495162,
    "test_time": 0,
    "response_time": 0,
    "base_url": "https://huggingface.co/Qwen/Qwen2.5-Omni-7B",
    "other": "",
    "balance": 0,
    "balance_updated_time": 0,
    "models": "m1,m2",
    "group": "default",
    "used_quota": 0,
    "model_mapping": "{\n  \"gpt-3.5-turbo\": \"gpt-3.5-turbo-0125\"\n}",
    "status_code_mapping": "",
    "priority": 1,
    "auto_ban": 1,
    "other_info": "",
    "settings": "",
    "tag": "test",
    "setting": "",
    "param_override": "{\"temperature\": 0\n}"
}
print(dict_to_model(data))


class Channel(BaseModel):
    id: Optional[int] = None
    name: str = 'name'
    type: int = '1'  # 8 自定义
    key: str = ''
    openai_organization: str = ''
    test_model: str = ''
    status: int = 1  # 0 禁用 1 启用

    priority: int = 0
    weight: int = 0

    test_time: int = 0
    response_time: int = '0'
    base_url: str = 'https://huggingface.co/Qwen/Qwen2.5-Omni-7B'
    other: str = ''
    balance: int = '0'
    balance_updated_time: int = '0'
    models: str = 'm1,m2'
    group: str = 'default'
    used_quota: int = '0'
    model_mapping: str = '{"gpt-3.5-turbo": "gpt-3.5-turbo-0125"}'
    status_code_mapping: str = ''
    auto_ban: int = '1'
    other_info: str = ''
    settings: str = ''
    tag: str = 'tag'
    setting: str = ''
    param_override: str = '{"temperature": 0}'

    created_time: int = '1745495162'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : filetype
# @Time         : 2023/7/15 15:05
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 文件或扩展名被更改

from meutils.pipe import *

import filetype
import magic
import mimetypes

import mimetypes

# mimetype, _ = mimetypes.guess_type('example.txt')
# print(mimetype)
#
# import magic
#
# file_type = magic.from_file('example.pdf', mime=True)
# print(file_type)
#
# import filetype
#
# kind = filetype.guess('example.jpg')
# if kind is None:
#     print('Cannot guess file type!')
# else:
#     print('File extension: %s' % kind.extension)
#     print('File MIME type: %s' % kind.mime)

if __name__ == '__main__':
    # mimetype, _ = mimetypes.guess_type('example.mp3')
    mimetype, _ = mimetypes.guess_type('https://www.baidu.com/img/flexible/logo/pc/result@2.png')

    print(mimetype)
    print(_)



import base64
import hashlib
import hmac
import time

import requests
import random

endpoint = "tmt.tencentcloudapi.com"
secret_id = "AKIDa9eDT1eYP5amxLnjme3KQrma6Vjp3gZM"
secret_key = "nlcNx68yc5QalkYBd1DmBPRIH9rNI3e3"


def get_string_to_sign(method, endpoint, params):
    s = method + endpoint + "/?"
    query_str = "&".join("%s=%s" % (k, params[k]) for k in sorted(params))
    return s + query_str


def sign_str(key, s, method):
    hmac_str = hmac.new(key.encode("utf8"), s.encode("utf8"), method).digest()
    return base64.b64encode(hmac_str)


def trans_tencent(q="苹果", fromLang='auto', toLang='en'):
    data = {
        'SourceText': q,
        'Source': fromLang,
        'Target': toLang,
        'Action': "TextTranslate",
        'Nonce': random.randint(32768, 65536),
        'ProjectId': 0,
        'Region': 'ap-hongkong',
        'SecretId': secret_id,
        'SignatureMethod': 'HmacSHA1',
        'Timestamp': int(time.time()),
        'Version': '2018-03-21',
    }
    s = get_string_to_sign("GET", endpoint, data)
    data["Signature"] = sign_str(secret_key, s, hashlib.sha1)

    # 此处会实际调用，成功后可能产生计费
    r = requests.get("https://" + endpoint, params=data, timeout=3)
    # print(r.json())
    return r.json()['Response']['TargetText']


if __name__ == '__main__':
    print(trans_tencent())
    print(trans_tencent('apple', 'auto', 'zh'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : 通用设计
# @Time         : 2024/11/25 18:13
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/GregaVrbancic/fastapi-celery/blob/master/app/main.py#L31

from meutils.pipe import *
from typing import Any, Dict, Optional
from celery.result import AsyncResult

from fastapi import FastAPI, APIRouter, Request, HTTPException
from enum import Enum
from pydantic import BaseModel


# 1. 定义标准响应模型
class TaskStatus(str, Enum):
    PENDING = "PENDING"
    SUCCESS = "SUCCESS"
    FAILURE = "FAILURE"
    RUNNING = "RUNNING"


class TaskResponse(BaseModel):
    code: int
    message: str
    task_id: str
    status: TaskStatus
    data: Optional[Any] = None


# 2. 定义任务管理器
class TaskManager:
    @staticmethod
    async def create_task(task_func, **kwargs) -> TaskResponse:
        try:
            task = task_func.delay(**kwargs)
            return TaskResponse(
                code=0,
                message="Task created successfully",
                task_id=task.id,
                status=TaskStatus.PENDING
            )
        except Exception as e:
            raise HTTPException(status_code=500, detail=str(e))

    @staticmethod
    async def get_task_result(task_id: str) -> TaskResponse:
        result = AsyncResult(id=task_id)

        if result.ready():
            if result.successful():
                return TaskResponse(
                    code=0,
                    message="Task completed successfully",
                    task_id=task_id,
                    status=TaskStatus.SUCCESS,
                    data=result.get()
                )
            else:
                return TaskResponse(
                    code=1,
                    message=str(result.result),
                    task_id=task_id,
                    status=TaskStatus.FAILURE
                )
        else:
            return TaskResponse(
                code=0,
                message="Task is still running",
                task_id=task_id,
                status=TaskStatus.RUNNING
            )


# 3. 路由实现
router = APIRouter(prefix="/celery")


@router.get("/{task_id}", response_model=TaskResponse)
async def get_result(task_id: str):
    return await TaskManager.get_task_result(task_id)


@router.post("/task", response_model=TaskResponse)
async def create_task(request: Request):
    try:
        kwargs = await request.json()
        return await TaskManager.create_task(proxy_task, **kwargs)
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))


# 4. 错误处理装饰器
def handle_task_errors(func):
    async def wrapper(*args, **kwargs):
        try:
            return await func(*args, **kwargs)
        except Exception as e:
            return TaskResponse(
                code=1,
                message=str(e),
                task_id="",
                status=TaskStatus.FAILURE
            )

    return wrapper


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()
    app.include_router(router)
    app.run(port=8899)



import pdfplumber
from pandas import DataFrame
import tabulate

"""
表格
"""


def curves_to_edges(cs):
    edges = []
    for c in cs:
        edges += pdfplumber.utils.rect_to_edges(c)
    return edges


def clean_cell_text(text):
    """
    清除文本中的换行符和多余空格
    """
    if text is None:
        return ""
    text = text.replace("\n", "")
    # 去除字符串开头和结尾的空白字符
    text = text.strip()
    return text


def extract_tables_with_text(self, pdf) -> List[str]:
    """抽取表格并嵌入文本"""

    def check_bboxes(word, table_bbox):
        left = word['x0'], word['top'], word['x1'], word['bottom']
        r = table_bbox
        return left[0] > r[0] and left[1] > r[1] and left[2] < r[2] and left[3] < r[3]

    lines = []
    for page in pdf.pages:
        tables = page.find_tables(
            table_settings={
                "vertical_strategy": "lines",
                "horizontal_strategy": "lines",
                "explicit_vertical_lines": self.curves_to_edges(page.curves) + page.edges,
                "explicit_horizontal_lines": self.curves_to_edges(page.curves) + page.edges,
            }
        )
        bboxes = [table.bbox for table in tables]
        tables = [{'table': i.extract(), 'top': i.bbox[1]} for i in tables]
        non_table_words = [word for word in page.extract_words() if
                           not any([check_bboxes(word, table_bbox) for table_bbox in bboxes])]

        for cluster in pdfplumber.utils.cluster_objects(non_table_words + tables, 'top', tolerance=5):
            if 'text' in cluster[0]:
                lines.append(' '.join([i['text'] for i in cluster]))
            elif 'table' in cluster[0]:
                lines.append(tabulate.tabulate(DataFrame(cluster[0]['table']).applymap(self.clean_cell_text),
                                               tablefmt="github"))
    return lines



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : n
# @Time         : 2024/12/6 17:51
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import json

from meutils.pipe import *
import httpx

BASE_URL = 'https://n.cn/api/common'
url = "/conversation/v2"

# Headers
headers = {
    'access-token': '12113771263956298908596801732877',
    'auth-token': 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJtaWQiOiIxMjExMzc3MTI2Mzk1NjI5ODkwODU5NjgwMTczMjg3NyIsInFpZCI6IiIsImRldGFpbCI6ImxvZ2luIiwiZXhwIjoxNzM0NzczODM0fQ.TE1UEoSSzWWPeEY8s3goiHuwNvLi6uCEu3r5WA7oop4',
    'chat-token': 'f3d6075b0fe2a4d0497c6a6cd0697c35D1vZ0xSNLxfIUnJqpvpDlb=/-ttIYxjLMb3EMrWwBZU1doBGbL3ZAVmzayK88v4PMHDv3pgnqKCjtMOX7wIXGmMLQdlGz/NxAfFOqMN98gNS+JMtNQ3cqfcQELgNPOnBwFJMk0=',
    'device-platform': 'Web',

    'request-id': '9d805c53-1fae-4db3-80f8-990b222747d2',

    'chat-date': 'Fri, 06 Dec 2024 09:59:53 GMT',  # 页面过期

    'sid': '12113771263956298908596801732877',
    'timestamp': '2024-12-06T17:59:53+08:00',
    'zm-token': 'a3c840bbc53ab84b1b91cd70a8a80b3e',
    'zm-ua': 'a455ebc67d0b5007e2a055414dd14d78',
    'zm-ver': '1.2',
    'Cookie': '__guid=12113771.405349637294117200.1732877020501.2803; webp=1; sdt=afb18c30-6534-4cb2-8c18-ade7636c7a47; _ga=GA1.1.2029555368.1732877022; Q=u%3D360H1374548850%26n%3DLhna_643%26le%3D%26m%3DZGt1WGWOWGWOWGWOWGWOWGWOZwZm%26qid%3D1374548850%26im%3D1_t01b9f6bfa3ea676416%26src%3Dpcw_namiso%26t%3D1; __NS_Q=u%3D360H1374548850%26n%3DLhna_643%26le%3D%26m%3DZGt1WGWOWGWOWGWOWGWOWGWOZwZm%26qid%3D1374548850%26im%3D1_t01b9f6bfa3ea676416%26src%3Dpcw_namiso%26t%3D1; T=s%3D65ecc6b3a47cc8e6df468f2246926051%26t%3D1732877071%26lm%3D0-1%26lf%3D2%26sk%3D93ed9bbe8cc2756ce12afad645453560%26mt%3D1732877071%26rc%3D%26v%3D2.0%26a%3D1; __NS_T=s%3D65ecc6b3a47cc8e6df468f2246926051%26t%3D1732877071%26lm%3D0-1%26lf%3D2%26sk%3D93ed9bbe8cc2756ce12afad645453560%26mt%3D1732877071%26rc%3D%26v%3D2.0%26a%3D1; __DC_sid=12113771.2545635413030612000.1733477831073.6633; Auth-Token=eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJtaWQiOiIxMjExMzc3MTI2Mzk1NjI5ODkwODU5NjgwMTczMjg3NyIsInFpZCI6IiIsImRldGFpbCI6ImxvZ2luIiwiZXhwIjoxNzM0NzczODM0fQ.TE1UEoSSzWWPeEY8s3goiHuwNvLi6uCEu3r5WA7oop4; __quc_silent__=1; test_cookie_enable=null; _ga_F1YB4HZHRB=GS1.1.1733477842.2.1.1733478071.14.0.1241444979; _ga_BCGTJC5JR6=GS1.1.1733477842.2.1.1733478071.0.0.0; __DC_monitor_count=12; _ga_MY08QYRPTL=GS1.1.1733477835.1.1.1733478388.31.0.1234880602; __DC_gid=12113771.779991695.1732877017079.1733479193697.18',
    'User-Agent': 'Apifox/1.0.0 (https://apifox.com)',
    'content-type': 'application/x-www-form-urlencoded;charset=UTF-8'
}

# Request data
data = {
    "conversation_id": "b18ae6a0b87a4b0f86f87ec2a36938c3",
    "message_id": "1864973026710818816",

    "re_answer_msg_id": "",
    "prompt": "周杰伦",
    "is_so": True,
    "is_copilot_enabled": False,
    "answer_mode": 4,
    "source_type": "prophet_web",
    "retry": False,
    "re_answer": 0,
    "search_type": "360",
    "model_type": "",
    "intention": "",
    "last_id": 0,
    "search_method": "1",
    "search_args": {},
    "msg_type": "",
    "kwargs": {
        "fr": "none",
        "zhuiwen": {
            "query": "周杰伦",
            "answer": "周杰伦",  # todo 追问拓展
        }
    }
}


# Make the request

async def create():
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=30) as client:
        payload = {
            "title": "周杰伦",
            "answer_mode": 4,
            "kwargs": {"fr": "none"}
        }
        response = await client.post("/conversation/v2", data=payload)
        logger.debug(response.json())

        _data = response.json()['data']

        data.update()
        logger.debug(data)
        data['conversation_id'] = _data['conversation_id']
        data['message_id'] = _data['msg_id']

        async with client.stream(method="POST", url="/chat/v2", json=data) as response:
            response.raise_for_status()
            logger.debug(response.status_code)

            # logger.debug(response.status_code)
            async for chunk in response.aiter_lines():
                # for chunk in "response.aiter_lines()":
                yield chunk


if __name__ == "__main__":
    arun(create())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : __init__.py
# @Time         : 2022/5/12 下午2:02
# @Author       : yuanjie
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


from meutils.pipe import *
from meutils.str_utils.regular_expression import parse_url, parse_command_string
from meutils.request_utils.crawler import Crawler
from urllib.parse import urlencode, parse_qs, parse_qsl, quote_plus, unquote_plus, urljoin

query_params2json = lambda q: dict(parse_qsl(q))


def parse_prompt(prompt, only_first_url: bool = True):
    if not only_first_url and (urls := parse_url(prompt)):
        prompt = reduce(lambda prompt, url: prompt.replace(url, " "), urls)
        return urls, prompt

    if prompt.startswith('http') and (prompts := prompt.split(maxsplit=1)):  # 单 url
        if len(prompts) == 2:
            return prompts
        else:
            return prompts + [' ']

    elif "http" in prompt and (urls := parse_url(prompt)):
        return urls[0], prompt.replace(urls[0], "")

    else:
        return None, prompt


def json2query_params(js: dict = None, url='') -> str:
    js = js or {}
    js = {k: v for k, v in js.items() if v}
    query_params = unquote_plus(urlencode(js))
    if query_params and url:
        url = f'{url}?{query_params}'
    return url


def remove_punctuation(sentence: str, punctuation='!"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~'):
    dic = str.maketrans('', '', punctuation)
    return sentence.translate(dic)


def chinese_convert(s, config='t2s'):
    """https://github.com/BYVoid/OpenCC
    pip install opencc
    pip install opencc-python-reimplemented

    s2t.json Simplified Chinese to Traditional Chinese 簡體到繁體
    t2s.json Traditional Chinese to Simplified Chinese 繁體到簡體
    s2tw.json Simplified Chinese to Traditional Chinese (Taiwan Standard) 簡體到臺灣正體
    tw2s.json Traditional Chinese (Taiwan Standard) to Simplified Chinese 臺灣正體到簡體
    s2hk.json Simplified Chinese to Traditional Chinese (Hong Kong variant) 簡體到香港繁體
    hk2s.json Traditional Chinese (Hong Kong variant) to Simplified Chinese 香港繁體到簡體
    s2twp.json Simplified Chinese to Traditional Chinese (Taiwan Standard) with Taiwanese idiom 簡體到繁體（臺灣正體標準）並轉換爲臺灣常用詞彙
    tw2sp.json Traditional Chinese (Taiwan Standard) to Simplified Chinese with Mainland Chinese idiom 繁體（臺灣正體標準）到簡體並轉換爲中國大陸常用詞彙
    t2tw.json Traditional Chinese (OpenCC Standard) to Taiwan Standard 繁體（OpenCC 標準）到臺灣正體
    hk2t.json Traditional Chinese (Hong Kong variant) to Traditional Chinese 香港繁體到繁體（OpenCC 標準）
    t2hk.json Traditional Chinese (OpenCC Standard) to Hong Kong variant 繁體（OpenCC 標準）到香港繁體
    t2jp.json Traditional Chinese Characters (Kyūjitai) to New Japanese Kanji (Shinjitai) 繁體（OpenCC 標準，舊字體）到日文新字體
    jp2t.json New Japanese Kanji (Shinjitai) to Traditional Chinese Characters (Kyūjitai) 日文新字體到繁體（OpenCC 標準，舊字體）
    tw2t.json Traditional Chinese (Taiwan standard) to Traditional Chinese 臺灣正體到繁體（OpenCC 標準）
    @return:
    """
    opencc = try_import("opencc", pip_name="opencc-python-reimplemented")
    converter = opencc.OpenCC(config)
    return converter.convert(s)


def sentence_cut(sentence, pattern='[，。！?\n]'):
    """'([，。！?\n])'可保留分隔符"""
    regexp = re.compile(pattern)

    return regexp.split(sentence.strip().strip('。').strip('！'))


def text2sentence(text):
    """断句
    https://github.com/blmoistawinde/HarvestText
    """
    _ = re.sub('([。！？\?])([^”’])', r"\1\n\2", text.strip())  # 单字符断句符
    _ = re.sub('(\.{6})([^”’])', r"\1\n\2", _)  # 英文省略号
    _ = re.sub('(\…{2})([^”’])', r"\1\n\2", _)  # 中文省略号
    _ = re.sub('([。！？\?][”’])([^，。！？\?])', r'\1\n\2', _)
    # 如果双引号前有终止符，那么双引号才是句子的终点，把分句符\n放到双引号后，注意前面的几句都小心保留了双引号
    # 很多规则中会考虑分号;，但是这里我把它忽略不计，破折号、英文双引号等同样忽略，需要的再做些简单调整即可。
    return _.rstrip().split("\n")


def half2all(ustring):
    """半角转全角"""
    rstring = ""
    for uchar in ustring:
        inside_code = ord(uchar)
        if inside_code == 32:  # 半角空格直接转化
            inside_code = 12288
        elif 32 <= inside_code <= 126:  # 半角字符（除空格）根据关系转化
            inside_code += 65248
        rstring += chr(inside_code)
    return rstring


def all2half(all_string):
    """全角转半角"""
    from idna import chr

    half_string = ""
    for char in all_string:
        inside_code = ord(char)
        if inside_code == 12288:  # 全角空格直接转换,全角和半角的空格的Unicode值相差12256
            inside_code = 32
        elif (inside_code >= 65281 and inside_code <= 65374):  # 全角字符（除空格）根据关系转化,除空格外的全角和半角的Unicode值相差65248
            inside_code -= 65248

        half_string += unichr(inside_code)
    return half_string


def to_hump(string="a_b c", pattern='_| '):
    """驼峰式转换"""
    reg = re.compile(pattern)
    _ = reg.sub('', string.title())
    return _.replace(_[0], _[0].lower())


def str_replace(s: str, dic: dict):
    """多值替换
        str_replace('abcd', {'a': '8', 'd': '88'})
    """
    return s.translate(str.maketrans(dic))


@lru_cache()
def arabic2chinese(arabic=123):
    c = Crawler(f'https://szjrzzwdxje.bmcx.com/{arabic}__szjrzzwdxje')
    return c.xpath('//span//text()')[-3:-1]


@lru_cache(1024)
def json_loads(s):
    if isinstance(s, bytes):
        s = s.decode()
    try:
        return json.loads(s.replace("'", '"'))

    except Exception as e:
        logger.warning(e)

        return eval(s)


class Encrypt(object):
    """加密"""

    def __init__(self, key=2):
        self.key = key

    def encrypt(self, s):
        b = bytearray(str(s).encode("utf-8"))
        n = len(b)
        c = bytearray(n * 2)
        j = 0
        for i in range(0, n):
            b1 = b[i]
            b2 = b1 ^ self.key
            c1 = b2 % 19
            c2 = b2 // 19
            c1 = c1 + 46
            c2 = c2 + 46
            c[j] = c1
            c[j + 1] = c2
            j = j + 2
        return c.decode("utf-8")

    def decrypt(self, s):
        c = bytearray(str(s).encode("utf-8"))
        n = len(c)
        if n % 2 != 0:
            return ""
        n = n // 2
        b = bytearray(n)
        j = 0
        for i in range(0, n):
            c1 = c[j]
            c2 = c[j + 1]
            j = j + 2
            c1 = c1 - 46
            c2 = c2 - 46
            b2 = c2 * 19 + c1
            b1 = b2 ^ self.key
            b[i] = b1
        return b.decode("utf-8")


def unicode_normalize(s):
    import unicodedata
    return unicodedata.normalize('NFKC', s)


def validate_url(url):
    if isinstance(url, list):
        return all(map(validate_url, url))

    # 首先检查 URL 格式
    try:
        parsed_url = urlparse(url)
        if not all([parsed_url.scheme, parsed_url.netloc]):
            return False  # , "URL 格式无效"

        # 检查格式是否符合标准
        if not re.match(r'^https?://', url):
            return False  # , "URL 必须以 http:// 或 https:// 开头"
    except Exception:

        logger.error("URL 解析错误")
        return False

    # 然后检查可访问性（可选）
    try:
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36'
        }

        # response = requests.head(url, timeout=5, allow_redirects=False, headers=headers)
        response = requests.head(url, timeout=5, headers=headers)
        if response.status_code >= 400:
            logger.error(f"URL 返回错误状态码: {response.status_code}")

            return False

        return True
    except requests.exceptions.RequestException as e:
        logger.error(f"连接错误: {str(e)}")

        return False

def parse_slice(slice_str):
    content = slice_str.strip()[1:-1]
    parts = content.split(':', 2)
    parts += [''] * (3 - len(parts))  # 确保有3部分
    return slice(
        int(parts[0]) if parts[0] else None,
        int(parts[1]) if parts[1] else None,
        int(parts[2]) if parts[2] else None
    )


if __name__ == '__main__':
    # print(str_replace('abcd', {'a': '8', 'd': '88'}))
    # print(unquote())
    # print(arabic2chinese())
    # print(to_hump())
    # print(translater())

    # print(json_loads("{1: 1}"))

    # print(Encrypt().encrypt('123' * 100))

    # print(query_params2json("act=get&uid=134597&vkey=07A1D82FDDE1E96DB5CEF4EF12C8125F&num=1&time=30&plat=1&re=1&type=0&so=1&ow=1&spl=1&addr=&db=1"))

    # print(half2all('⽉月'))
    # print(all2half('⽉月'))

    print(parse_prompt(" hi", only_first_url=False))

    print(parse_prompt("https://www.hao123.com/ hi"))
    print(parse_prompt("https://www.hao123.com/ hi" * 2))
    print(parse_prompt("hi https://www.hao123.com/ "))

    print(parse_prompt("https://www.hao123.com/ hi" * 2, only_first_url=False))

    # import chardet
    #
    # def detect_encoding(byte_content):
    #     result = chardet.detect(byte_content)
    #     return result['encoding']

    url = 'https://fal.ai/models/fal-ai/flux-pro/kontext/requests/de5f28be-2ca8-4bd4-8c42-c7fc32969801?output=0'
    url = "https://5b0988e595225.cdn.sohucs.com/images/20190814/5ebb727f502545718c4a06f199cd848b.jpeg"
    # url = "https://filesystem.site/cdn/20250609/1XPdqIyhHiOJ8SC68W4ZQGBrf7XRZD.png"

    # url = "https://filesystem.site/cdn/20250609/1QUKzDHRQedraO15CXnUc22aBjvqEN.png"

    # url = "https://photog.art/api/oss/R2yh8N"

    url = "https://p3-bot-workflow-sign.byteimg.com/tos-cn-i-mdko3gqilj/1fe07cca46224208bfbed8c0f3c50ed8.png~tplv-mdko3gqilj-image.image?rk3s=81d4c505&x-expires=1780112531&x-signature=e7q1NOMjqCHvMz%2FC3dVAEVisAh4%3D&x-wf-file_name=9748f6214970f744fe7fd7a3699cfa2.png"

    # print(validate_url([url] * 3))
    print(validate_url(url))

    print(re.findall("http", url*2))

    # 示例用法
    slice_str = "[:2]"
    slice_str = ":2]"

    # slice_str="https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=Y7HVfo[:2]"
    # https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=Y7HVfo%5B%3A2%5D
    slice_obj = parse_slice(slice_str)
    l = list(range(10))
    print(l[slice_obj])  # [0, 1]




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : st
# @Time         : 2025/4/3 15:24
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.async_utils import async_to_sync

from meutils.io.files_utils import to_url, to_bytes
from meutils.schemas.openai_types import CompletionRequest

from google import genai
from google.genai.types import HttpOptions, GenerateContentConfig, Content, HarmCategory, HarmBlockThreshold, Part, \
    ThinkingConfig
from google.genai.chats import _is_part_type

config = GenerateContentConfig(

    temperature=0.7,
    top_p=0.8,
    response_modalities=['Text', 'Image'],
)

# self._http_options.base_url = 'https://generativelanguage.googleapis.com/'
# self._http_options.api_version = 'v1beta'
client = genai.Client(
    api_key="AIzaSyCa8PYURpxFKz7yOtQB_O_wRfrX0gYh9L4",
    http_options=HttpOptions(
        base_url="https://all.chatfire.cc/genai"
    )
)

GOOGLE_CREDENTIALS_JSON="""{
  "type": "service_account",
  "project_id": "mystic-gradient-460414-s3",
  "private_key_id": "074dc3f29316a8d663c6af1eb3e9b60f9c440230",
  "private_key": "-----BEGIN PRIVATE KEY-----\nMIIEvgIBADANBgkqhkiG9w0BAQEFAASCBKgwggSkAgEAAoIBAQCtAW0wwjk0GheH\nF1PEps5Q+smR9Fc9lBcK6R5cTpJo0aNC/9xvz6yqFXYpTjTqwUYK8EH62RFP4K9g\neZRTAQdF4qXP7S1cPsyeBEtk2whsjUIThS0dK1FZzOM3CLM+JRUU9AJdkQWuY4pT\ngfcZ+x5KtfbbTZQ3rf91ZPbaJID/cEd1yjiquhSWG6UGYxN09YPEmPOMAUSBmphI\nWhPdMLih3lghxHRbIhXF2y8uTAIVifyT4xFb7C9IT4NphMt/Ee5hpN+GzlveNsPK\n6UdO7XBgpWX+GjxCoxKgvuvpsJxBprv87BVar4jOFS8YaIMBaSKV37ZKDs1/JNaa\nYn0nCepJAgMBAAECggEABCMNOH3idoqbrqsqAYEwHItcq9DtD5/Fi8kehNxLn7ZI\nmkLqQ2GyyifkeREhzo3D1iHf4AbWBUQVCYBqwa/b+8mzR5UvMR4e0DX/1AfxTAA2\nOOeFuEV3hudRdRjQCW/DUOqTTme3/C8s6PmJ/jztIOH4Rs70eP7gBY9+ICIAlCsR\n258PYn3AkLq6u3f0lYVeafV2I52aX861FlQTGNfTg3Npixx0ygWKrppX67bhlzm8\nyedpkCc3S+ZIjfYAG8mO57vYWUi0oPA6TDbzAQBHqq1Su2+WvsCaqpJ3PtPVigSj\n/KSsUmCXGMXDl3/yRBvuzV6SMlpylBx72SJY8Pn0gQKBgQDz9f6+58l7BBG1fFKe\nYJdRViuGjNG6pKfXnPFaZEpwwgJx2C4chkEKBB6ge6hiIrk9CCE365xjVl82l99x\nqukOWf6dsW6nX7U4oMNQ53ZMPpCdHvaFI9E/B9Exl01GGx61Ajx88NMFVXDYbWa+\n+gKZ6DFeMukMwqWAPqRmMbp8gQKBgQC1iwn6mbsxiSteZ9SzfE+rPb9FkxC/W0VO\nQMBwTVD52M5STZbuXj+oRP1DXFEz1HWNWhrX2w92faLgaZQv5rbXXXkI5zD1Ymlo\n82yjxjoe2fs79GzuQjs5pVzr9Gd7qjqDiHQETV67uqkjjjrBDxtnh2pBJrQ7NAX4\nWQ5YvO2pyQKBgQDlWaLAftAf4ldzswlI863OPaB1wfV57koMnfMY86g4EWyS8pWH\nI4AJlo3kedx3RgW3pujz1YEUsqKYhHcAOZkYcOq5/ZD1sTiBoEF+0duZGkiCLhW8\nXUveYc79PCzdd6mKqiuLmiwIm6aXEWLCr++6GVwwHtSRtnw48KtFmAKmAQKBgEHn\n6AeCJ+Mj2Qq8xwsh5gpUsEuhyEKv78ko2LPsXalWfBCE+nYu4Q1Ffmwl55TZXJOk\nUnpqnIJ3udmeep2pNFebJ4WD6NIsl/fx507Nd+wbVom9go48nSOgVMuUPr02X2fA\n8d1d+TZN3Xl6+Pl9coVzM6/TTTrw9/ng2sR8UguBAoGBAJ6SMKAqa+yjZezDUkC3\nUkmywTRX9vfhH5mDv+kj1MHW++hLYhLE7WRNtMhgf9idLEQBbw7jzoS4wmx9jtPu\nP0mUh67fjctbtR6uxbuZl5sY3eMx03hmv5H33aazhMA7mQXjMPS7gXTIxMoE2Wmw\nFpJyddFAuwMDZecDr74xNU9b\n-----END PRIVATE KEY-----\n",
  "client_email": "xxxxx-350@mystic-gradient-460414-s3.iam.gserviceaccount.com",
  "client_id": "115968049812880579488",
  "auth_uri": "https://accounts.google.com/o/oauth2/auth",
  "token_uri": "https://oauth2.googleapis.com/token",
  "auth_provider_x509_cert_url": "https://www.googleapis.com/oauth2/v1/certs",
  "client_x509_cert_url": "https://www.googleapis.com/robot/v1/metadata/x509/xxxxx-350%40mystic-gradient-460414-s3.iam.gserviceaccount.com",
  "universe_domain": "googleapis.com"
}"""


client = genai.Client(
    vertexai=True,
    credentials=GOOGLE_CREDENTIALS_JSON,
    http_options=HttpOptions(
        base_url="https://all.chatfire.cc/genai"
    )
)
if __name__ == '__main__':
    model = "gemini-2.0-flash-exp-image-generation"
    model = "gemini-2.5-flash-preview-04-17"

    request = CompletionRequest(
        model=model,
        messages=[
            {
                "role": "user", "content": [
                {
                    "type": "text",
                    "text": "一句话总结"
                },
                {
                    "type": "image_url",
                    "image_url": {

                        "url": "https://oss.ffire.cc/files/kling_watermark.png"

                    }
                }
            ]
            }],

    )

    response = client.aio.chats.create(
        model=model,
        config=GenerateContentConfig(
            system_instruction=None,
            max_output_tokens=None,
            # response_modalities=['Text', 'Image'],

            thinking_config=ThinkingConfig(include_thoughts=True, thinking_budget=24576),
        ),

        # history=[
        #
        #     Content(
        #         role="user",
        #         parts=[
        #             Part.from_text(
        #                 text="画条狗"
        #             )
        #         ]
        #     ),
        #     Content(
        #         role="model",
        #         parts=[
        #             Part.from_text(
        #                 text="Ok"
        #             ),
        #
        #             Part.from_bytes(
        #                 data=async_to_sync(to_bytes)("https://oss.ffire.cc/files/kling_watermark.png"),
        #                 mime_type="image/png"
        #             ),
        #         ]
        #     )
        #
        # ]
    )

    # "mime_type": "image/png"

    p = Part.from_uri(file_uri="https://oss.ffire.cc/files/kling_watermark.png", mime_type="image/png")
    pp = Part.from_bytes(
        data=async_to_sync(to_bytes)("https://oss.ffire.cc/files/kling_watermark.png"),
        mime_type="image/png"
    )


    # pt = Part.from_text(text="文本")

    # {'video_metadata': None,
    #  'thought': None,
    #  'code_execution_result': None,
    #  'executable_code': None,
    #  'file_data': {'file_uri': 'https://oss.ffire.cc/files/kling_watermark.png',
    #                'mime_type': 'image/png'},
    #  'function_call': None,
    #  'function_response': None,
    #  'inline_data': None,
    #  'text': None}

    #
    async def main():
        message = [
            Part.from_text(text="1+1"),
            # pp
        ]

        print(_is_part_type(message))
        chunks = await response.send_message_stream(message)

        async for chunk in chunks:
            logger.debug(chunk)
            if chunk.candidates:
                parts = chunk.candidates[0].content.parts
                for part in parts or []:
                    if len(str(part)) < 500:
                        logger.debug(part)
                    if part.inline_data:
                        image_url = await to_url(part.inline_data.data, mime_type=part.inline_data.mime_type)
                        logger.info(image_url)

                    if part.text:
                        logger.info(part.text)
            # print(i.model_dump_json(indent=4, exclude_none=True))  # inline_data


    # response = client.models.generate_content(
    #     model="gemini-2.0-flash-exp-image-generation",
    #     contents=['画条狗'],
    #
    #     # model="gemini-2.5-pro-exp-03-25",
    #     # model="gemini-2.0-flash",
    #
    #     # contents=[
    #     #     Part.from_uri(file_uri='https://generativelanguage.googleapis.com/v1beta/files/test', mime_type='image/png'),
    #     #
    #     #           "一句话总结"],
    #     config=config
    # )
    #
    # # client.aio.
    # # client.aio.chats.create()
    #
    # if __name__ == '__main__':
    #     arun(file_object)

    arun(main())

# ValueError: Message must be a valid part type: typing.Union[google.genai.types.File, google.genai.types.Part, PIL.Image.Image, str] or typing.Union[google.genai.types.File, google.genai.types.Part, PIL.Image.Image, str, google.genai.types.PartDict], got <class 'list'>

"""
curl "https://all.chatfire.cc/genai/v1beta/models/gemini-2.5-flash-preview-04-17:generateContent?key=AIzaSyCa8PYURpxFKz7yOtQB_O_wRfrX0gYh9L4" \
-H 'Content-Type: application/json' \
-X POST \
-d '{
  "contents": [
    {
      "parts": [
        {
          "text": "9.8 9.11 哪个大"
        }
      ]
    }
  ],
  "generationConfig": {
    "thinkingConfig": {
        "includeThoughts": true,
          "thinkingBudget": 1024
    }
  }
}'
"""

from litellm import completion
import os

# auth: run 'gcloud auth application-default'
os.environ["VERTEXAI_PROJECT"] = "hardy-device-386718"
os.environ["VERTEXAI_LOCATION"] = "us-central1"

response = completion(
  model="vertex_ai/gemini-1.5-pro",
  messages=[{ "content": "Hello, how are you?","role": "user"}]
)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : utils
# @Time         : 2025/7/4 17:34
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import httpx

from meutils.pipe import *
from meutils.apis.utils import make_request
from meutils.io.files_utils import to_base64
from urllib.parse import urlencode, quote

base_url = "https://image.baidu.com/aigc"


async def pic_upload(
        image: Union[str, bytes],
        token: Optional[str] = None,
):
    # data = {
    #     "token": token or "2df26",
    #     "scene": "pic_edit",
    #     "picInfo": quote(await to_base64(image)),
    #     "timestamp": 1751605857221,
    #     # "pageFr": None
    # }

    token = token or "2df26"
    image = await to_base64(image, content_type='image/png')
    # logger.debug(image)
    image = quote(image)
    data = f'token={token}&scene=pic_edit&timestamp=1751605857221&picInfo={image}&pageFr='
    # logger.debug(data)
    #
    # data = urlencode(data)
    # logger.debug(data)
    headers = {
        # 'User-Agent': 'Apifox/1.0.0 (https://apifox.com)',
        'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8'
    }

    response = requests.post(base_url, data=data, headers=headers)
    response.raise_for_status()
    print(response.text)
    return response.json()

    # async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=100) as client:
    #     response = await client.post("/pic_upload", data=data)
    #     response.raise_for_status()
    #
    #
    #     return response.json()


async def upload(image):
    # image = await to_base64(image, content_type='image/png')
    # image = quote(image)

    import requests

    url = "https://image.baidu.com/aigc/pic_upload"

    payload = f'token=fbf4c&scene=pic_edit&picInfo={image}&timestamp=1751622550663&pageFr='

    # data = {
    #     "token": "fbf4c",
    #     "scene": "pic_edit",
    #     "picInfo": await to_base64(image),
    #     "timestamp": 1751605857221,
    #     "pageFr": None
    # }
    # payload = urlencode(data)
    headers = {
        'User-Agent': 'Apifox/1.0.0 (https://apifox.com)',
        'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8'
    }

    response = requests.request("POST", url, headers=headers, data=payload)

    print(response.text)

if __name__ == '__main__':
    image = "/Users/betterme/PycharmProjects/AI/img.png"

    # arun(to_base64(image))
    # arun(pic_upload(image))

    arun(upload(image))

    # print(urlencode({"base:xx": ":"}))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : ann_v1
# @Time         : 2021/4/17 12:22 下午
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  : pymilvus==1.0.1
import copy
from meutils.pipe import *
from milvus import Milvus, MetricType, IndexType


class ANN(object):

    def __init__(self, host='HOST', port='19530', pool="SingletonThread"):
        self.host = host
        self.client = Milvus(
            host, port,
            handler='GRPC' if port == '19530' else 'HTTP',  # 19530, 19121
            pool=pool,  # 线程池
            pool_size=32,
        )
        logger.info(f"ServerVersion {self.client.server_version()}")
        logger.info(f"ClientVersion {self.client.client_version()}")

    def __getattr__(self, collection_name) -> Collection:
        return Collection(collection_name, self.client)

    def create_collection(self, collection_name='TEST',
                          dim=768,
                          index_file_size=1024,
                          metric_type='IP',
                          index_type='IVF_FLAT',
                          nlist=4096,  # 4*n**0.5
                          partitions=None,
                          auto_id=True,
                          overwrite=True,
                          sleep=6):

        if self.client.has_collection(collection_name):
            if overwrite:
                logger.warning(f"{collection_name} already exists! to drop.")
                self.client.drop_collection(collection_name, timeout=300)
                time.sleep(sleep)  # "Collection already exists and it is in delete state, please wait a second"
            else:
                return f"{collection_name} already exists!"

        # 建表
        params = {
            'collection_name': collection_name,
            'dimension': dim,
            'metric_type': MetricType.__getattr__(metric_type),
            'index_type': IndexType.__getattr__(index_type),
            'index_file_size': index_file_size,
            'index_param': {'nlist': nlist},
            'auto_id': auto_id
        }

        self.client.create_collection(params)
        self.client.create_index(collection_name, params['index_type'], params['index_param'])

        # 分区：删除分区就不需要重新建表
        if partitions is not None:
            for part in partitions:
                self.client.create_partition(collection_name, partition_tag=part)

        logger.info(f"{self.client.get_collection_info(collection_name)}")


class Collection(object):

    def __init__(self, name=None, client=None):
        self.name = name
        self.client = client

    def __str__(self):
        _, ok = self.client.has_collection(self.name)
        if not ok:
            logger.warning(f"{self.name}  doesn't exist")
        return f"Collection({self.name})"

    def insert(self, vectors, batch_size=100000, partition_tag=None):
        # todo: 多进程插入
        ids = []
        n = max(len(vectors) // batch_size, 1)

        for a in tqdm(np.array_split(vectors, n)):
            _, _ids = self.client.insert(self.name, a, partition_tag=partition_tag)
            ids += _ids

        return ids

    def search(self, vectors, topk=10, nprobe=64, partition_tags=None, threshold=None, union=True):
        status, results = self.client.search(
            self.name, topk, vectors,
            partition_tags=partition_tags,
            params={"nprobe": nprobe},
        )

        if threshold:
            dfs = [pd.DataFrame(map(lambda x: x.__dict__, r))[lambda df: df.distance > threshold] for r in results]
        else:
            dfs = [pd.DataFrame(map(lambda x: x.__dict__, r)) for r in results]

        if dfs:
            return pd.concat(dfs) if union else dfs  # id, distance
        else:
            return pd.DataFrame(columns=['id', 'distance'])

    def create_partition(self, partition_tag):
        return self.client.drop_partition(self.name, partition_tag)

    def drop_partition(self, partition_tag):
        return self.client.drop_partition(self.name, partition_tag)

    def get_entity_by_id(self, ids):
        return self.client.get_entity_by_id(self.name, ids)

    def delete_entity_by_id(self, ids):
        return self.client.delete_entity_by_id(self.name, ids)

    @property
    def count(self):
        return self.client.count_entities(self.name)

    @property
    def info(self):
        return self.client.get_collection_info(self.name)[1]

    @property
    def stats(self):
        return self.client.get_collection_stats(self.name)[1]

    @property
    def partitions(self):
        return self.client.list_partitions(self.name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : common
# @Time         : 2024/5/6 08:52
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : httpx重试 transport = httpx.AsyncHTTPTransport(retries=3) # response.raise_for_status()

from meutils.pipe import *
from meutils.caches import rcache
from meutils.str_utils import parse_slice
from meutils.decorators.retry import retrying
from meutils.decorators.contextmanagers import try_catcher
from meutils.notice.feishu import send_message
from meutils.db.redis_db import redis_client, redis_aclient
from typing import Optional, Union

from urllib.parse import urlparse, parse_qs, unquote

FEISHU_BASE_URL = "https://open.feishu.cn/open-apis/"


def get_app_access_token():
    payload = {
        "app_id": os.getenv("FEISHU_APP_ID", "cli_a60cfd9cad76100b"),
        "app_secret": os.getenv("FEISHU_APP_SECRET")
    }
    response = httpx.post(
        "https://open.feishu.cn/open-apis/auth/v3/app_access_token/internal",
        json=payload,
        timeout=30,
    )

    # logger.debug(response.json())

    return response.json().get("app_access_token")


@retrying(max_retries=3, predicate=lambda x: not x)
async def aget_app_access_token():
    """不加缓存，access_token会提前失效
    该报错是因为开发者本次调用过程中使用的 Tenant Access Token 已经失效或有误，飞书开放平台无法判断当前请求是否来自一个可信的用户，因此拦截了情况。
    :return:
    """
    payload = {
        "app_id": os.getenv("FEISHU_APP_ID"),
        "app_secret": os.getenv("FEISHU_APP_SECRET")
    }
    # logger.debug(payload)
    kwargs = {
        # "proxy": np.random.choice(["http://110.42.51.223:38443", None])
    }
    async with httpx.AsyncClient(base_url=FEISHU_BASE_URL, timeout=30, **kwargs) as client:
        response = await client.post("/auth/v3/app_access_token/internal", json=payload)

        return response.is_success and response.json().get("app_access_token")  # False / None


def get_spreadsheet_values(
        spreadsheet_token: Optional[str] = None,
        sheet_id: Optional[str] = None,
        feishu_url=None,
        to_dataframe: Optional[bool] = False
):
    if feishu_url and feishu_url.startswith("http"):
        parsed_url = urlparse(feishu_url)
        spreadsheet_token = parsed_url.path.split('/')[-1]
        sheet_id = parsed_url.query.split('=')[-1]

    url = f"https://open.feishu.cn/open-apis/sheets/v2/spreadsheets/{spreadsheet_token}/values/{sheet_id}"

    headers = {
        "Authorization": f"Bearer {get_app_access_token()}"
    }
    response = httpx.get(url, headers=headers, timeout=30)
    _ = response.json()
    return _ if not to_dataframe else pd.DataFrame(_.get('data').get('valueRange').get('values'))


@alru_cache(ttl=600)
@rcache(ttl=300, serializer='pickle')  # 缓存
async def aget_spreadsheet_values(
        spreadsheet_token: Optional[str] = None,
        sheet_id: Optional[str] = None,
        feishu_url: Optional[str] = None,
        to_dataframe: Optional[bool] = False
):
    if feishu_url and feishu_url.startswith("http"):
        parsed_url = urlparse(feishu_url)
        spreadsheet_token = parsed_url.path.split('/')[-1]
        sheet_id = parsed_url.query.split('=')[-1]

    access_token = await aget_app_access_token()
    headers = {
        "Authorization": f"Bearer {access_token}"
    }
    async with httpx.AsyncClient(base_url=FEISHU_BASE_URL, timeout=30, headers=headers) as client:
        response = await client.get(f"/sheets/v2/spreadsheets/{spreadsheet_token}/values/{sheet_id}")
        if response.is_success:
            data = response.json()
            if to_dataframe:
                values = data.get('data').get('valueRange').get('values')

                return pd.DataFrame(values)

            return data
        else:
            send_message(
                f"{response.status_code}\n\n{access_token}\n\n{response.text}",
                '飞书为啥为none: 已经去掉缓存了'
            )
            return get_spreadsheet_values(spreadsheet_token, sheet_id, feishu_url, to_dataframe)


async def spreadsheet_values_append(
        spreadsheet_token: Optional[str] = None,
        sheet_id: Optional[str] = None,
        feishu_url=None,
        range: Optional[str] = None,
        values: Optional[list] = None
):
    """ https://open.feishu.cn/document/server-docs/docs/sheets-v3/data-operation/append-data

        https://open.feishu.cn/open-apis/sheets/v2/spreadsheets/:spreadsheetToken/values_prepend
        https://open.feishu.cn/open-apis/sheets/v2/spreadsheets/:spreadsheetToken/values_append
        https://open.feishu.cn/open-apis/sheets/v2/spreadsheets/:spreadsheetToken/values_image

    :param spreadsheet_token:
    :param sheet_id:
    :param feishu_url:
    :param values:
    :return:
    """

    if feishu_url and feishu_url.startswith("http"):
        parsed_url = urlparse(feishu_url)
        spreadsheet_token = parsed_url.path.split('/')[-1]
        sheet_id = parsed_url.query.split('=')[-1]

    access_token = await aget_app_access_token()
    headers = {
        "Authorization": f"Bearer {access_token}",
        "Content-Type": "application/json; charset=utf-8",

    }
    payload = {
        "valueRange": {
            # https://open.feishu.cn/document/server-docs/docs/sheets-v3/overview
            # "range": f"{sheet_id}!A:B",  # <sheetId>!<开始列>:<结束列>
            # "range": f"{sheet_id}!A1:E3",  # <sheetId>!<开始单元格>:<结束列>
            # "range": f"{sheet_id}",  # 非空的最大行列范围内的数据

            "range": f"{sheet_id}!{range}" if range else sheet_id,
            "values": values
        }
    }

    async with httpx.AsyncClient(base_url=FEISHU_BASE_URL, timeout=30, headers=headers) as client:
        # 默认覆盖 ?insertDataOption=OVERWRITE
        response = await client.post(f"/sheets/v2/spreadsheets/{spreadsheet_token}/values_append", json=payload)
        return response.is_success and response.json() or response.text


def create_document(title: str = "一篇新文档🔥", folder_token: Optional[str] = None):
    payload = {
        "title": title,
        "folder_token": folder_token,
    }

    url = "https://open.feishu.cn/open-apis/docx/v1/documents"
    headers = {
        "Authorization": f"Bearer {get_app_access_token()}"
    }
    response = httpx.post(url, headers=headers, timeout=30, json=payload)
    return response.json()


def get_doc_raw_content(document_id: str = "BxlwdZhbyoyftZx7xFbcGCZ8nah"):
    url = f"https://open.feishu.cn/open-apis/docx/v1/documents/{document_id}/raw_content"
    headers = {
        "Authorization": f"Bearer {get_app_access_token()}"
    }
    response = httpx.get(url, headers=headers, timeout=30)
    return response.json()


async def get_next_from_redis(redis_key):
    if api_key := await redis_aclient.lpop(redis_key):
        await redis_aclient.rpush(redis_key, api_key)
        return api_key


async def get_next_token(
        feishu_url,
        check_token: Optional[Callable] = None,
        min_points: float = 0,  # vidu min_points=4
        ttl: Optional[int] = None,
):
    _ttl = await redis_aclient.ttl(feishu_url)  # 列表操作会取消ttl
    for i in range(10):  # 避免死循环
        if i > 5:
            send_message(f"重复同步了{i}次\n\n{feishu_url}", title="更新tokens")

        if token := await redis_aclient.lpop(feishu_url):
            token = token.decode()

            if check_token is None:
                logger.info("写回队列「跳过核验」")

                await redis_aclient.rpush(feishu_url, token)
                _ttl > 1 and await redis_aclient.expire(feishu_url, _ttl)

                return token
            elif await check_token(token, threshold=min_points * 2):
                logger.info("写回队列「大于 2x消耗」")  # 大于最小消耗 至少两次 才写回

                await redis_aclient.rpush(feishu_url, token)
                _ttl > 1 and await redis_aclient.expire(feishu_url, _ttl)

                return token
            elif await check_token(token):
                logger.info("不写回队列「最后一次消耗」")
                return token

        else:  # 更新tokens到redis
            df = await aget_spreadsheet_values(feishu_url=feishu_url, to_dataframe=True)
            # api_keys = [k for k in df[0] if isinstance(k, str)]  # 过滤空值, 避免 list： 一般超链接造成的
            api_keys = []
            for api_key in df[0]:
                if api_key:
                    if isinstance(api_key, str):
                        api_keys.append(api_key)
                    else:
                        api_key = "".join(map(lambda x: x.get('text', ""), api_key))
                        api_keys.append(api_key)

            # 初始化
            if check_token:
                bools = await asyncio.gather(*map(check_token, api_keys))  # todo: 提前过滤一遍, 减少初始化次数
                api_keys = list(itertools.compress(api_keys, bools))

            if not api_keys:  # 全部失效
                logger.debug(f"全部无效：{feishu_url}")
                break

            await redis_aclient.delete(feishu_url)
            num = await redis_aclient.rpush(feishu_url, *api_keys)
            send_message(f"新增：{num}\n\n{feishu_url}", title="更新tokens")

            if ttl:  ########### 实际上并未生效 每次操作都会变
                _ = await redis_aclient.expire(feishu_url, ttl)  # Redis 的 RPUSH 命令本身不支持直接设置过期时间


async def get_next_token_for_polling(
        feishu_url,
        check_token: Optional[Callable] = None,
        max_size: Optional[int] = 10,
        from_redis: Optional[bool] = False,
        min_points: float = 0,

        ttl: Optional[int] = None,
):
    if from_redis:
        return await get_next_token(feishu_url, check_token, ttl=ttl or 1 * 24 * 3600, min_points=min_points)

        # 轮询
    max_size = max_size or 10000
    df = await aget_spreadsheet_values(feishu_url=feishu_url, to_dataframe=True)

    api_keys = []
    for api_key in df[0]:
        if api_key:
            if isinstance(api_key, str):
                api_keys.append(api_key)
            else:
                logger.debug("feishu表格数据: 非字符串")
                api_key = "".join(map(lambda x: x.get('text', ""), api_key))
                api_keys.append(api_key)

    api_keys = np.random.choice(api_keys, size=min(max_size, len(api_keys)))

    num = len(api_keys)
    logger.debug(f"重复数：{num - len(set(list(api_keys)))}")

    if check_token:
        with try_catcher("CHECK_TOKEN"):
            tasks = map(check_token, api_keys)
            api_keys = [k for k, v in zip(api_keys, await asyncio.gather(*tasks)) if v]

            prob = len(api_keys) / num
            logger.debug(f"过滤前：{num} 过滤后：{len(api_keys)}，存活率：{prob:.2%}")
            if prob < 0.3:
                send_message(f"存活率 ≤ {prob}，及时更新", title=__name__)  # todo: 提醒

    return np.random.choice(api_keys)


@alru_cache(ttl=3600)
async def get_dataframe(iloc_tuple: Optional[tuple] = None, feishu_url: Optional[str] = None, ):  # 系统配置
    feishu_url = feishu_url or "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=bl4jYm"
    df = await aget_spreadsheet_values(feishu_url=feishu_url, to_dataframe=True)

    if iloc_tuple:
        return df.iloc._getitem_tuple(iloc_tuple)  # df.iloc._getitem_tuple((0, 1))
    return df


async def get_series(feishu_url: str, index: int = 0, duplicated: bool = False):  # 系统配置
    # 前置处理
    # https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=Y7HVfo[:2]
    slice_obj = None
    if feishu_url.endswith(']') and '[' in feishu_url:  # 是否解码
        logger.debug(feishu_url)
        feishu_url, slice_string = feishu_url.rsplit('[', maxsplit=1)
        slice_obj = parse_slice(f"[{slice_string}")

    df = await aget_spreadsheet_values(feishu_url=feishu_url, to_dataframe=True)
    series = df[index]
    values = [i for i in series if i and isinstance(i, str)]  # todo: 非标准字符串处理
    if duplicated:  # 有序去重
        values = values | xUnique

    if slice_obj:
        values = values[slice_obj]
    return values


if __name__ == '__main__':
    # print(get_app_access_token())
    # print(get_spreadsheet_values("Qy6OszlkIhwjRatkaOecdZhOnmh", "0f8eb3"))
    # pprint(get_spreadsheet_values("Bmjtst2f6hfMqFttbhLcdfRJnNf", "Y9oalh"))
    # pd.DataFrame(
    #     get_spreadsheet_values("Bmjtst2f6hfMqFttbhLcdfRJnNf", "79272d").get('data').get('valueRange').get('values'))

    # print(get_doc_raw_content("TAEFdXmzyobvgUxKM3lcLfc2nxe"))
    # print(create_document())
    # "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=79272d"

    # r = get_spreadsheet_values(feishu_url="https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=79272d",
    #                            to_dataframe=True)
    # print(list(filter(None, r[0])))
    # print(get_spreadsheet_values("Bmjtst2f6hfMqFttbhLcdfRJnNf", "79272d"))

    print(arun(aget_app_access_token()))
    # df = arun(aget_spreadsheet_values(
    #     feishu_url="https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=5i64gO",
    #     to_dataframe=True
    #
    # ))
    # print(df)

    from inspect import iscoroutinefunction

    # print(filter(lambda x: x and x.strip(), df[0]) | xlist)

    # func = aget_app_access_token()

    # print(alru_cache(ttl=300)(sync_to_async(aget_app_access_token))())

    # for i in tqdm(range(10)):
    #     # print(aget_app_access_token())
    #     print(arun(aget_app_access_token()))
    #     # print(get_app_access_token())

    # values = [
    #     [
    #         "2023/12/25",
    #         "收入",
    #         "微信",
    #         "100",
    #         "帐号 老表max"
    #     ],
    #     [
    #         "2023/12/25",
    #         "支出",
    #         "支付宝",
    #         "10",
    #         "买东西 老表max"
    #     ],
    #     [
    #         "2023/12/26",
    #         "支出",
    #         "支付宝",
    #         "19.9",
    #         "买东西 老表max"
    #     ],
    # ]
    #
    # _ = spreadsheet_values_append(
    #     feishu_url="https://xchatllm.feishu.cn/sheets/BPxjsmNj7hZr7Ytwk9uccvOKn6Z?sheet=7ce4e3",
    #
    #     range="A1:E3",
    #     values=values
    #
    # )
    #
    # print(arun(_))
    #
    # print(create_document())
    # arun(get_dataframe(iloc_tuple=(1, 0)))
    # feishu_url = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=EYgZ8c"
    # feishu_url = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=V84y6z"
    # with timer():
    #     token = arun(get_next_token(feishu_url))

    # FEISHU_URL = "https://xchatllm.feishu.cn/sheets/XfjqszII3hZAEvtTOgecOgv2nye?sheet=c14b34"
    FEISHU_URL = "https://xchatllm.feishu.cn/sheets/RIv6sAUtFhlZYItyYa6ctdv1nvb?sheet=0bcf4a[:2]"
    FEISHU_URL = "https://xchatllm.feishu.cn/sheets/RIv6sAUtFhlZYItyYa6ctdv1nvb?sheet=0bcf4a"
    FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=Gvm9dt[:30]"
    FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Z59Js10DbhT8wdt72LachSDlnlf?sheet=ydUVB1"

    # FEISHU_URL="https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=Y7HVfo%5B%3A2%5D"

    # r = arun(get_series(FEISHU_URL))

    # print(redis_client.expire(FEISHU_URL, 1))
    # print(redis_client.ttl(FEISHU_URL))

    # print(redis_client.lpop(FEISHU_URL))

    arun(get_next_token(FEISHU_URL, ttl=100))

    # print(redis_client.ttl("xx"))
